#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_INTELLIGENTATOM_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_INTELLIGENTATOM_H

#include <unordered_map>
#include <string>
#include <stdexcept>
#include <tuple>

#include "InteractingAtom.h"


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class PeriodicLiquid;

			class IntelligentAtom :public InteractingAtom
			{
				using size_type = std::size_t;
				using NumericVector = NumericalAnalysis::LinearAlgebra::NumericVector;
				using NumericMatrix = NumericalAnalysis::LinearAlgebra::NumericMatrix;
				using CrystalLattice = SolidStatePhysics::Materials::CrystalLattice;

				using AtomicAdaptor = SolidStatePhysics::Crystallization::Calculators::TranslatingAdaptor<InteractingAtom>;

// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors, destructor, and operator

			public:
				IntelligentAtom(const size_type atomicNumber);
				IntelligentAtom(const size_type atomicNumber, const NumericVector& cartesianCoordinate);
				virtual ~IntelligentAtom() = default;

				IntelligentAtom(const IntelligentAtom&);
				IntelligentAtom(IntelligentAtom&&) noexcept;
				IntelligentAtom& operator=(const IntelligentAtom&);
				IntelligentAtom& operator=(IntelligentAtom&&) noexcept;

			// Constructors, destructor, and operator
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

				bool hasIdealCoordinationNumber() const noexcept override;
				bool isAnion() const noexcept override;
				bool isCation() const noexcept override;
				size_type maxCommonBridgingAtomsSharing() const noexcept override;
				std::pair<size_type, double> bondingMotivation() const noexcept override;

				void resetBondingMotivation(const std::pair<size_type, double>& bondingMotivation);

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void initialize() noexcept override;
				std::string getFingerprint(const double maxBondingLengthErrorRate) const noexcept;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				bool m_isAnion;
				size_type m_maxCommonBridgingAtomsSharing;
				std::pair<size_type, double> m_bondingMotivation;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline bool SolidStatePhysics::Crystallization::Components::IntelligentAtom::hasIdealCoordinationNumber() const noexcept
{
	if (getCoordinationNumber() == m_bondingMotivation.first)
		return true;
	else
		return false;
}

inline bool SolidStatePhysics::Crystallization::Components::IntelligentAtom::isAnion() const noexcept
{
	return m_isAnion;
}

inline bool SolidStatePhysics::Crystallization::Components::IntelligentAtom::isCation() const noexcept
{
	return !(m_isAnion);
}

inline SolidStatePhysics::Crystallization::Components::IntelligentAtom::size_type SolidStatePhysics::Crystallization::Components::IntelligentAtom::maxCommonBridgingAtomsSharing() const noexcept
{
	return m_maxCommonBridgingAtomsSharing;
}

inline std::pair<SolidStatePhysics::Crystallization::Components::IntelligentAtom::size_type, double> SolidStatePhysics::Crystallization::Components::IntelligentAtom::bondingMotivation() const noexcept
{
	return m_bondingMotivation;
}

inline void SolidStatePhysics::Crystallization::Components::IntelligentAtom::resetBondingMotivation(const std::pair<size_type, double>& bondingMotivation)
{
	m_bondingMotivation = bondingMotivation;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline std::string SolidStatePhysics::Crystallization::Components::IntelligentAtom::getFingerprint(const double maxBondingLengthErrorRate) const noexcept
{
	std::string fingerprintString;
	{
		fingerprintString += std::string{ "AN_" };
		fingerprintString += std::to_string(atomicNumber());
		fingerprintString += std::string{ "_CB_" };
		fingerprintString += getChemicalBondsKey();
		fingerprintString += std::string{ "_BAS_" };
		fingerprintString += getCommonBridgingAtomsSharingsKey();
		fingerprintString += std::string{ "_IC_" };
		fingerprintString += getIonicConstraintsKey(maxBondingLengthErrorRate);
	}

	return fingerprintString;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_INTELLIGENTATOM_H
