#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_IONICCONSTRAINTER_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_IONICCONSTRAINTER_H

#include <string>
#include <stdexcept>
#include <functional>

#include "MinimumConstrainter.h"
#include "ConstraintOwners.h"


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class IonicConstrainter :public SolidStatePhysics::Crystallization::Calculators::MinimumConstrainter
			{
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors and destructor

			public:
				IonicConstrainter(IonicConstraintOwner& realOwner, const IonicConstraintOwnerAdaptor& attachedOwner);
				IonicConstrainter(IonicConstraintOwner& realOwner, IonicConstraintOwnerAdaptor&& attachedOwner);
				explicit IonicConstrainter(const std::shared_ptr<IonicConstrainter>& twinPtr);

				virtual ~IonicConstrainter();

			// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Static methods

				static void create(IonicConstraintOwner& realOwner, const IonicConstraintOwnerAdaptor& attachedOwner);
				static void erase(std::shared_ptr<IonicConstrainter>&& ionicInteractionConstrainterPtr);

				static bool shouldCreate(const std::shared_ptr<InteratomicConnector>&) noexcept;
				static bool shouldCreate(const InteractingAtom& ownerAtom, const AtomicAdaptor& attachedAtom) noexcept;

				static double formationRangeCoef() noexcept;
				static void resetFormationRangeCoef(const double formationRangeCoef);

			// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void renew() override;
				bool isFeasible(const double maxOverlapErrorRate) const noexcept;

				IonicConstraintOwner& realOwner() const noexcept;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::reference_wrapper<IonicConstraintOwner> m_realOwner;
				IonicConstraintOwnerAdaptor m_attachedOwner;

				static double m_formationRangeCoef;


			private:
				IonicConstrainter() = delete;
				IonicConstrainter(const IonicConstrainter&) = delete;
				IonicConstrainter(IonicConstrainter&&) noexcept = delete;
				IonicConstrainter& operator=(const IonicConstrainter&) = delete;
				IonicConstrainter& operator=(IonicConstrainter&&) noexcept = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

inline void SolidStatePhysics::Crystallization::Components::IonicConstrainter::create(IonicConstraintOwner& realOwner, const IonicConstraintOwnerAdaptor& attachedOwner)
{
	InteratomicConnector::createInteratomicComponent(realOwner, attachedOwner);
}

inline void SolidStatePhysics::Crystallization::Components::IonicConstrainter::erase(std::shared_ptr<IonicConstrainter>&& ionicConstrainterPtr)
{
	InteratomicConnector::eraseInteratomicComponent(std::move(ionicConstrainterPtr));
}

inline double SolidStatePhysics::Crystallization::Components::IonicConstrainter::formationRangeCoef() noexcept
{
	return m_formationRangeCoef;
}

inline void SolidStatePhysics::Crystallization::Components::IonicConstrainter::resetFormationRangeCoef(const double formationRangeCoef)
{
	if (0.0 < formationRangeCoef)
		m_formationRangeCoef = formationRangeCoef;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"resetFormationRangeCoef\" in the class of SolidStatePhysics::Crystallization::Components::IonicConstrainter.\n";
		errorMessage += "Message : \"formationRangeCoef\" is not more than zero.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void SolidStatePhysics::Crystallization::Components::IonicConstrainter::renew()
{
	renewMinimumDistance(ownerAtom().maximumIonicRadius() + attachedAtom().entity().maximumIonicRadius());
}

inline bool SolidStatePhysics::Crystallization::Components::IonicConstrainter::isFeasible(const double maxOverlapErrorRate) const noexcept
{
	double minBondLength = (1.0 - maxOverlapErrorRate) * minimumDistance();

	if (interatomicDistance() < minBondLength)
		return false;
	else
		return true;
}

inline SolidStatePhysics::Crystallization::Components::IonicConstraintOwner& SolidStatePhysics::Crystallization::Components::IonicConstrainter::realOwner() const noexcept
{
	return m_realOwner.get();
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_IONICCONSTRAINTER_H
