#ifndef SOLIDSTATEPHYSICS_UNITSYSTEM_LENGTHUNIT_H
#define SOLIDSTATEPHYSICS_UNITSYSTEM_LENGTHUNIT_H

#include <ratio>


namespace SolidStatePhysics
{
	namespace UnitSystem
	{
		namespace LengthUnit
		{
			struct AtomicUnit
			{
				static constexpr double unitLengthInSI() noexcept;
			};

			struct Angstrom
			{
				static constexpr double unitLengthInSI() noexcept;
			};

			template <typename Magnitude = std::ratio<1>>
			struct Metre
			{
				static constexpr double unitLengthInSI() noexcept;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constant expressions

inline constexpr double SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit::unitLengthInSI() noexcept
{
	return ((0.0529177210903 * static_cast<double>(std::nano::num)) / static_cast<double>(std::nano::den));
}

inline constexpr double SolidStatePhysics::UnitSystem::LengthUnit::Angstrom::unitLengthInSI() noexcept
{
	return ((0.1 * static_cast<double>(std::nano::num)) / static_cast<double>(std::nano::den));
}

template <typename Magnitude>
inline constexpr double SolidStatePhysics::UnitSystem::LengthUnit::Metre<Magnitude>::unitLengthInSI() noexcept
{
	return (static_cast<double>(Magnitude::num) / static_cast<double>(Magnitude::den));
}

// Constant expressions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_UNITSYSTEM_LENGTHUNIT_H
