#ifndef NUMERICALANALYSIS_LINEARALGEBRA_LINEAREQUATIONSOLVER_H
#define NUMERICALANALYSIS_LINEARALGEBRA_LINEAREQUATIONSOLVER_H

#include <memory>

#include <mkl_lapacke.h>

#include "NumericMatrix.h"
#include "NumericVector.h"


namespace NumericalAnalysis
{
	namespace LinearAlgebra
	{
		inline NumericVector solveLinearEquation(NumericMatrix& nm, NumericVector& nv)
		{
			NumericVector resultNumericVector;

			if (nm.columnSize() == nm.rowSize() && nm.rowSize() == nv.size())
			{
				std::size_t order = nm.columnSize();
				std::unique_ptr<lapack_int> ipiv{ new lapack_int[order] };

				if (LAPACKE_dgetrf(LAPACK_COL_MAJOR, order, order, nm.begin(), order, ipiv.get()) == 0)
				{
					if (LAPACKE_dgetrs(LAPACK_COL_MAJOR, 'N', order, 1, nm.begin(), order, ipiv.get(), nv.begin(), order) == 0)
					{
						nm.clear();
						nv.swap(resultNumericVector);
						return resultNumericVector;
					}
				}				
			}

			return resultNumericVector;
		}

		inline NumericMatrix solveLinearEquation(NumericMatrix& nma, NumericMatrix& nmb)
		{
			NumericMatrix resultNumericMatrix;

			if (nma.columnSize() == nma.rowSize() && nma.rowSize() == nmb.columnSize())
			{
				std::size_t order = nma.columnSize();
				std::unique_ptr<lapack_int> ipiv{ new lapack_int[order] };

				if (LAPACKE_dgetrf(LAPACK_COL_MAJOR, order, order, nma.begin(), order, ipiv.get()) == 0)
				{
					if (LAPACKE_dgetrs(LAPACK_COL_MAJOR, 'N', order, nmb.rowSize(), nma.begin(), order, ipiv.get(), nmb.begin(), order) == 0)
					{
						nma.clear();
						nmb.swap(resultNumericMatrix);
						return resultNumericMatrix;
					}
				}
			}

			return resultNumericMatrix;
		}
	}
}



#endif // !NUMERICALANALYSIS_LINEARALGEBRA_LINEAREQUATIONSOLVER_H
