#ifndef SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_METALLICCONSTRAINTER_H
#define SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_METALLICCONSTRAINTER_H

#include <string>
#include <stdexcept>
#include <functional>

#include "MinimumConstrainter.h"
#include "ConstraintOwners.h"


namespace SolidStatePhysics
{
	namespace Crystallization
	{
		namespace Components
		{
			class MetallicConstrainter :public SolidStatePhysics::Crystallization::Calculators::MinimumConstrainter
			{
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructors and destructor

			public:
				MetallicConstrainter(MetallicConstraintOwner& realOwner, const MetallicConstraintOwnerAdaptor& attachedOwner);
				MetallicConstrainter(MetallicConstraintOwner& realOwner, MetallicConstraintOwnerAdaptor&& attachedOwner);
				explicit MetallicConstrainter(const std::shared_ptr<MetallicConstrainter>& twinPtr);

				virtual ~MetallicConstrainter();

			// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Static methods

				static void create(MetallicConstraintOwner& realOwner, const MetallicConstraintOwnerAdaptor& attachedOwner);
				static void erase(std::shared_ptr<MetallicConstrainter>&& ionicInteractionConstrainterPtr);

				static bool shouldCreate(const std::shared_ptr<InteratomicConnector>&) noexcept;
				static bool shouldCreate(const InteractingAtom& ownerAtom, const AtomicAdaptor& attachedAtom) noexcept;

				static double formationRangeCoef() noexcept;
				static void resetFormationRangeCoef(const double formationRangeCoef);

			// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Public methods

				void renew() override;
				bool isFeasible(const double maxOverlapErrorRate) const noexcept;

				MetallicConstraintOwner& realOwner() const noexcept;

			// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				std::reference_wrapper<MetallicConstraintOwner> m_realOwner;
				MetallicConstraintOwnerAdaptor m_attachedOwner;

				static double m_formationRangeCoef;


			private:
				MetallicConstrainter() = delete;
				MetallicConstrainter(const MetallicConstrainter&) = delete;
				MetallicConstrainter(MetallicConstrainter&&) noexcept = delete;
				MetallicConstrainter& operator=(const MetallicConstrainter&) = delete;
				MetallicConstrainter& operator=(MetallicConstrainter&&) noexcept = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

inline void SolidStatePhysics::Crystallization::Components::MetallicConstrainter::create(MetallicConstraintOwner& realOwner, const MetallicConstraintOwnerAdaptor& attachedOwner)
{
	InteratomicConnector::createInteratomicComponent(realOwner, attachedOwner);
}

inline void SolidStatePhysics::Crystallization::Components::MetallicConstrainter::erase(std::shared_ptr<MetallicConstrainter>&& metallicConstrainterPtr)
{
	InteratomicConnector::eraseInteratomicComponent(std::move(metallicConstrainterPtr));
}

inline double SolidStatePhysics::Crystallization::Components::MetallicConstrainter::formationRangeCoef() noexcept
{
	return m_formationRangeCoef;
}

inline void SolidStatePhysics::Crystallization::Components::MetallicConstrainter::resetFormationRangeCoef(const double formationRangeCoef)
{
	if (0.0 < formationRangeCoef)
		m_formationRangeCoef = formationRangeCoef;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"resetFormationRangeCoef\" in the class of SolidStatePhysics::Crystallization::Components::MetallicConstrainter.\n";
		errorMessage += "Message : \"formationRangeCoef\" is not more than zero.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline void SolidStatePhysics::Crystallization::Components::MetallicConstrainter::renew()
{
	renewMinimumDistance(ownerAtom().metallicRadius() + attachedAtom().entity().metallicRadius());
}

inline bool SolidStatePhysics::Crystallization::Components::MetallicConstrainter::isFeasible(const double maxOverlapErrorRate) const noexcept
{
	double minBondLength = (1.0 - maxOverlapErrorRate) * minimumDistance();

	if (interatomicDistance() < minBondLength)
		return false;
	else
		return true;
}

inline SolidStatePhysics::Crystallization::Components::MetallicConstraintOwner& SolidStatePhysics::Crystallization::Components::MetallicConstrainter::realOwner() const noexcept
{
	return m_realOwner.get();
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !SOLIDSTATEPHYSICS_CRYSTALLIZATION_COMPONENTS_METALLICCONSTRAINTER_H
