#ifndef CONTROLCENTER_SYSTEM_MPICONTROLLER_H
#define CONTROLCENTER_SYSTEM_MPICONTROLLER_H

#include <string>


namespace ControlCenter
{
	namespace System
	{
		class MpiController final
		{
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		private:
			MpiController() = default;
			~MpiController() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

		public:
			static std::size_t thisMpiNumber() noexcept;
			static std::size_t totalMpiCount() noexcept;
			static const std::string& thisMpiProcessName() noexcept;

			static void resetThisMpiNumber(const std::size_t thisMpiNumber) noexcept;
			static void resetTotalMpiCount(const std::size_t totalMpiCount) noexcept;

			static bool isLeadingMpi() noexcept;
			static bool hasValidMpiConstants() noexcept;

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			static std::size_t m_thisMpiNumber;
			static std::size_t m_totalMpiCount;
			static std::string m_thisMpiProcessName;

		private:
			MpiController(const MpiController&) = delete;
			MpiController(MpiController&&) noexcept = delete;
			MpiController& operator=(const MpiController&) = delete;
			MpiController& operator=(MpiController&&) noexcept = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::size_t ControlCenter::System::MpiController::thisMpiNumber() noexcept
{
	return m_thisMpiNumber;
}

inline std::size_t ControlCenter::System::MpiController::totalMpiCount() noexcept
{
	return m_totalMpiCount;
}

inline const std::string& ControlCenter::System::MpiController::thisMpiProcessName() noexcept
{
	return m_thisMpiProcessName;
}

inline void ControlCenter::System::MpiController::resetThisMpiNumber(const std::size_t thisMpiNumber) noexcept
{
	m_thisMpiNumber = thisMpiNumber;

	m_thisMpiProcessName = std::string{ "Process_" };
	m_thisMpiProcessName += std::to_string(m_thisMpiNumber);
}

inline void ControlCenter::System::MpiController::resetTotalMpiCount(const std::size_t totalMpiCount) noexcept
{
	m_totalMpiCount = totalMpiCount;
}

inline bool ControlCenter::System::MpiController::isLeadingMpi() noexcept
{
	return !(static_cast<bool>(m_thisMpiNumber));
}

inline bool ControlCenter::System::MpiController::hasValidMpiConstants() noexcept
{
	if (m_thisMpiNumber < m_totalMpiCount)
		return true;
	else
		return false;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_SYSTEM_MPICONTROLLER_H
