#ifndef CONTROLCENTER_WINDOW_RECEPTIONIST_H
#define CONTROLCENTER_WINDOW_RECEPTIONIST_H

#include <stdexcept>
#include <string>
#include <filesystem>
#include <set>
#include <utility>

#include "OrderForm.h"
#include "CrystallizationPlan.h"
#include "MonitoringPlan.h"


namespace ControlCenter
{
	namespace Window
	{
		class Receptionist final
		{
			using size_type = std::size_t;
			using CrystallizationVariables = SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables;
			using CrystallizationPlan = SolidStatePhysics::Crystallization::Simulators::CrystallizationPlan;
			using MonitoringPlan = SolidStatePhysics::Crystallization::Simulators::MonitoringPlan;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructor and destructor

		public:
			Receptionist() = default;
			~Receptionist() = default;

		// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Public methods

			OrderForm createOrderForm(const std::filesystem::path& orderSheetPath);
			void sendOrderSheetManual();
			void sendCommandLineArgumentsManual();

		// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods

		private:
			std::string loadOrderSheet(const std::filesystem::path& orderSheetPath) const;

			void readExecutionEnvironment(const std::string& orderSheetText);
			void readIOPaths(const std::string& orderSheetText);
			void readMonitoringPlan(const std::string& orderSheetText);

			void readVertexSharingCationicPairs(const std::string& orderSheetText);
			void readEdgeSharingCationicPairs(const std::string& orderSheetText);
			void readFaceSharingCationicPairs(const std::string& orderSheetText);
			void readCommonOptimizationVariables(const std::string& orderSheetText);
			void readGlobalOptimizationVariables(const std::string& orderSheetText);
			void readLocalOptimizationVariables(const std::string& orderSheetText);
			void readRequestedFeedstocks(const std::string& orderSheetText);

		// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Private methods for reading methods

			CrystallizationVariables readCrystallizationVariables(const std::string& inputText);
			std::set<std::pair<std::string, std::string>> readCationicPairs(const std::string& inputText);

			bool getSwitch(const std::string& flag);

		// Private methods for reading methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			mutable OrderForm m_orderForm;


		private:
			Receptionist(const Receptionist&) = delete;
			Receptionist(Receptionist&&) noexcept = delete;
			Receptionist& operator=(const Receptionist&) = delete;
			Receptionist& operator=(Receptionist&&) noexcept = delete;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

inline ControlCenter::Window::OrderForm ControlCenter::Window::Receptionist::createOrderForm(const std::filesystem::path& orderSheetPath)
{
	m_orderForm.clear();
	std::string orderSheetText = loadOrderSheet(orderSheetPath);

	readExecutionEnvironment(orderSheetText);
	readIOPaths(orderSheetText);
	readMonitoringPlan(orderSheetText);
	readVertexSharingCationicPairs(orderSheetText);
	readEdgeSharingCationicPairs(orderSheetText);
	readFaceSharingCationicPairs(orderSheetText);
	readCommonOptimizationVariables(orderSheetText);
	readGlobalOptimizationVariables(orderSheetText);
	readLocalOptimizationVariables(orderSheetText);
	readRequestedFeedstocks(orderSheetText);

	if (m_orderForm.isValid())
	{
		OrderForm orderForm = m_orderForm;
		m_orderForm.clear();

		return orderForm;
	}
	
	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"createOrderForm\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read the order form successfully.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods for reading methods

inline bool ControlCenter::Window::Receptionist::getSwitch(const std::string& flag)
{
	if (flag == "ON" || flag == "On" || flag == "on")
		return true;

	else if (flag == "OFF" || flag == "Off" || flag == "off")
		return false;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"getSwitch\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read \"flag\".\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

// Private methods for reading methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_WINDOW_RECEPTIONIST_H
