#ifndef CONTROLCENTER_ADMINISTERING_REQUESTEDFEEDSTOCKS_H
#define CONTROLCENTER_ADMINISTERING_REQUESTEDFEEDSTOCKS_H

#include <stdexcept>
#include <string>
#include <vector>

#include "ChemicalConstituents.h"


namespace ControlCenter
{
	namespace Administering
	{
		class RequestedFeedstocks final
		{
			using size_type = std::size_t;
			using ChemicalConstituents = SolidStatePhysics::Materials::ChemicalConstituents;

// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Constructors, destructor, and operators

		public:
			RequestedFeedstocks();
			~RequestedFeedstocks() = default;

			RequestedFeedstocks(const RequestedFeedstocks&) = default;
			RequestedFeedstocks(RequestedFeedstocks&&) noexcept = default;
			RequestedFeedstocks& operator=(const RequestedFeedstocks&) = default;
			RequestedFeedstocks& operator=(RequestedFeedstocks&&) noexcept = default;

		// Constructors, destructor, and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Access methods

			size_type feedstocksPerChemicalConstituents() const;
			const std::vector<ChemicalConstituents>& targetChemicalConstituents() const;

			void resetFeedstocksPerChemicalConstituents(const size_type feedstocksPerChemicalConstituents);
			void resetTargetChemicalConstituents(const std::vector<ChemicalConstituents>& targetChemicalConstituents);
			void resetTargetChemicalConstituents(std::vector<ChemicalConstituents>&& targetChemicalConstituents);

		// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
		// Utility

			std::string getRequestName() const;

			bool isValid() const noexcept;
			void clear();

		// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************

		private:
			size_type m_feedstocksPerChemicalConstituents;
			std::vector<ChemicalConstituents> m_targetChemicalConstituents;
		};
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline std::size_t ControlCenter::Administering::RequestedFeedstocks::feedstocksPerChemicalConstituents() const
{
	if (isValid())
		return m_feedstocksPerChemicalConstituents;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"feedstocksPerChemicalConstituents\" in the class of ControlCenter::Administering::RequestedFeedstocks.\n";
		errorMessage += "Message : This is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

inline const std::vector<SolidStatePhysics::Materials::ChemicalConstituents>& ControlCenter::Administering::RequestedFeedstocks::targetChemicalConstituents() const
{
	if (isValid())
		return m_targetChemicalConstituents;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"targetChemicalConstituents\" in the class of ControlCenter::Administering::RequestedFeedstocks.\n";
		errorMessage += "Message : This is invalid.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

inline void ControlCenter::Administering::RequestedFeedstocks::resetFeedstocksPerChemicalConstituents(const size_type feedstocksPerChemicalConstituents)
{
	if (feedstocksPerChemicalConstituents != 0)
		m_feedstocksPerChemicalConstituents = feedstocksPerChemicalConstituents;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"resetFeedstocksPerChemicalConstituents\" in the class of ControlCenter::Administering::RequestedFeedstocks.\n";
		errorMessage += "Message : \"feedstocksPerChemicalConstituents\" is zero.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

inline void ControlCenter::Administering::RequestedFeedstocks::resetTargetChemicalConstituents(const std::vector<ChemicalConstituents>& targetChemicalConstituents)
{
	if (targetChemicalConstituents.empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"resetTargetChemicalConstituents\" in the class of ControlCenter::Administering::RequestedFeedstocks.\n";
		errorMessage += "Message : \"targetChemicalConstituents\" is empty.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}

	else
		m_targetChemicalConstituents = targetChemicalConstituents;
}

inline void ControlCenter::Administering::RequestedFeedstocks::resetTargetChemicalConstituents(std::vector<ChemicalConstituents>&& targetChemicalConstituents)
{
	if (targetChemicalConstituents.empty())
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member function of \"resetTargetChemicalConstituents\" in the class of ControlCenter::Administering::RequestedFeedstocks.\n";
		errorMessage += "Message : targetChemicalConstituentsCollection is empty.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}

	else
		m_targetChemicalConstituents = std::move(targetChemicalConstituents);
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

inline bool ControlCenter::Administering::RequestedFeedstocks::isValid() const noexcept
{
	if (m_targetChemicalConstituents.empty() || m_feedstocksPerChemicalConstituents == 0)
		return false;
	else
		return true;
}

inline void ControlCenter::Administering::RequestedFeedstocks::clear()
{
	m_feedstocksPerChemicalConstituents = 0;
	m_targetChemicalConstituents.clear();
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_ADMINISTERING_REQUESTEDFEEDSTOCKS_H
