#ifndef CONTROLCENTER_SYSTEM_IO_SYSTEMPATHSREPOSITORY_H
#define CONTROLCENTER_SYSTEM_IO_SYSTEMPATHSREPOSITORY_H

#include <filesystem>


namespace ControlCenter
{
	namespace System
	{
		namespace IO
		{
			class SystemPathsRepository final
			{
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Constructor and destructor

			private:
				SystemPathsRepository() = default;
				~SystemPathsRepository() = default;

			// Constructor and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Access methods

			public:
				static const std::filesystem::path& inputDirectoryPath() noexcept;
				static const std::filesystem::path& outputDirectoryPath() noexcept;
				static const std::filesystem::path& workDirectoryPath() noexcept;

				static void resetInputDirectoryPath(const std::filesystem::path& inputDirectoryPath);
				static void resetOutputDirectoryPath(const std::filesystem::path& outputDirectoryPath);
				static void resetWorkDirectoryPath(const std::filesystem::path& workDirectoryPath);

			// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
			// Methods

				static void tryInitializeOutputDirectory();
				static void tryInitializeWorkDirectory();

				static void transportAllDataFromWorkToOutputDirectory();

			// Methods
// **********************************************************************************************************************************************************************************************************************************************************************************************

			private:
				static std::filesystem::path m_inputDirectoryPath;
				static std::filesystem::path m_outputDirectoryPath;
				static std::filesystem::path m_workDirectoryPath;

			private:
				SystemPathsRepository(const SystemPathsRepository&) = delete;
				SystemPathsRepository(SystemPathsRepository&&) noexcept = delete;
				SystemPathsRepository& operator=(const SystemPathsRepository&) = delete;
				SystemPathsRepository& operator=(SystemPathsRepository&&) noexcept = delete;
			};
		}
	}
}

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access methods

inline const std::filesystem::path& ControlCenter::System::IO::SystemPathsRepository::inputDirectoryPath() noexcept
{
	return m_inputDirectoryPath;
}

inline const std::filesystem::path& ControlCenter::System::IO::SystemPathsRepository::outputDirectoryPath() noexcept
{
	return m_outputDirectoryPath;
}

inline const std::filesystem::path& ControlCenter::System::IO::SystemPathsRepository::workDirectoryPath() noexcept
{
	return m_workDirectoryPath;
}

// Access methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************

#endif // !CONTROLCENTER_SYSTEM_IO_SYSTEMPATHSREPOSITORY_H
