#include "AbstractChemicalElement.h"

#include <regex>
#include <limits>

#include "UnitConversion.h"

using namespace SolidStatePhysics::Crystallization::Calculators;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructor

AbstractChemicalElement::AbstractChemicalElement()
	: m_isAnion{ false }
	, m_bondingAffinity{ 0.0 }
	, m_coordinationNumber{ 0 }
	, m_maxCommonBridgingAtomsSharing{ 0 }
	, m_possibleMaxCoordinationNumber{ 0 }
{
}

// Constructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Static methods

std::string AbstractChemicalElement::getElementSymbol(const size_type elementNumber)
{
	switch (elementNumber)
	{
	case 8:
		return std::string{ "O" };

	case 11:
		return std::string{ "Na" };

	case 13:
		return std::string{ "Al" };
	case 14:
		return std::string{ "Si" };
	case 15:
		return std::string{ "P" };
	case 16:
		return std::string{ "S" };

	case 19:
		return std::string{ "K" };
	case 20:
		return std::string{ "Ca" };
	case 21:
		return std::string{ "Sc" };

	case 22:
		return std::string{ "Ti" };

	case 26:
		return std::string{ "Fe" };
	case 27:
		return std::string{ "Co" };
	case 28:
		return std::string{ "Ni" };
	case 29:
		return std::string{ "Cu" };
	case 30:
		return std::string{ "Zn" };
	case 31:
		return std::string{ "Ga" };

	case 39:
		return std::string{ "Y" };
	case 40:
		return std::string{ "Zr" };
	case 41:
		return std::string{ "Nb" };
	case 42:
		return std::string{ "Mo" };
	case 43:
		return std::string{ "Tc" };
	case 44:
		return std::string{ "Ru" };
	case 45:
		return std::string{ "Rh" };
	case 46:
		return std::string{ "Pd" };
	case 47:
		return std::string{ "Ag" };

	case 49:
		return std::string{ "In" };

	case 57:
		return std::string{ "La" };
	case 58:
		return std::string{ "Ce" };
	case 59:
		return std::string{ "Pr" };
	case 60:
		return std::string{ "Nd" };
	case 61:
		return std::string{ "Pm" };
	case 62:
		return std::string{ "Sm" };
	case 63:
		return std::string{ "Eu" };
	case 64:
		return std::string{ "Gd" };
	case 65:
		return std::string{ "Tb" };
	case 66:
		return std::string{ "Dy" };
	case 67:
		return std::string{ "Ho" };
	case 68:
		return std::string{ "Er" };
	case 69:
		return std::string{ "Tm" };
	case 70:
		return std::string{ "Yb" };
	case 71:
		return std::string{ "Lu" };

	case 94:
		return std::string{ "Pu" };

	default:
		break;
	}


	std::string errorMessage = "------------------------------------------------------\n\n";
	errorMessage += "Error   : Invalid argument\n";
	errorMessage += "Location: The member funtion of \"getElementSymbol\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
	errorMessage += "Message : \"elementNumber\" is out of definitions.\n\n";
	errorMessage += "------------------------------------------------------\n\n";
	throw std::invalid_argument(errorMessage);
}

AbstractChemicalElement::size_type AbstractChemicalElement::getElementNumber(const std::string& elementSymbol)
{
	if (elementSymbol == "O")
		return 8;
	else if (elementSymbol == "S")
		return 16;


	else if (elementSymbol == "Na")
		return 11;

	else if (elementSymbol == "Si")
		return 13;
	else if (elementSymbol == "Si")
		return 14;
	else if (elementSymbol == "P")
		return 15;

	else if (elementSymbol == "K")
		return 19;
	else if (elementSymbol == "Ca")
		return 20;
	else if (elementSymbol == "Sc")
		return 21;

	else if (elementSymbol == "Ti")
		return 22;

	else if (elementSymbol == "Fe")
		return 26;
	else if (elementSymbol == "Co")
		return 27;
	else if (elementSymbol == "Ni")
		return 28;
	else if (elementSymbol == "Cu")
		return 29;
	else if (elementSymbol == "Zn")
		return 30;
	else if (elementSymbol == "Ga")
		return 31;

	else if (elementSymbol == "Y")
		return 39;
	else if (elementSymbol == "Zr")
		return 40;
	else if (elementSymbol == "Nb")
		return 41;
	else if (elementSymbol == "Mo")
		return 42;
	else if (elementSymbol == "Tc")
		return 43;
	else if (elementSymbol == "Ru")
		return 44;
	else if (elementSymbol == "Rh")
		return 45;
	else if (elementSymbol == "Pd")
		return 46;
	else if (elementSymbol == "Ag")
		return 47;

	else if (elementSymbol == "In")
		return 49;

	else if (elementSymbol == "La")
		return 57;
	else if (elementSymbol == "Ce")
		return 58;
	else if (elementSymbol == "Pr")
		return 59;
	else if (elementSymbol == "Nd")
		return 60;
	else if (elementSymbol == "Pm")
		return 61;
	else if (elementSymbol == "Sm")
		return 62;
	else if (elementSymbol == "Eu")
		return 63;
	else if (elementSymbol == "Gd")
		return 64;
	else if (elementSymbol == "Tb")
		return 65;
	else if (elementSymbol == "Dy")
		return 66;
	else if (elementSymbol == "Ho")
		return 67;
	else if (elementSymbol == "Er")
		return 68;
	else if (elementSymbol == "Tm")
		return 69;
	else if (elementSymbol == "Yb")
		return 70;
	else if (elementSymbol == "Lu")
		return 71;

	else if (elementSymbol == "Pu")
		return 94;

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member funtion of \"getElementNumber\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
		errorMessage += "Message : \"elementSymbol\" is out of definitions.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Static methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void AbstractChemicalElement::resetElementNumber(const size_type elementNumber)
{
	std::string elementSymbol = getElementSymbol(elementNumber);
	{
		if (elementSymbol.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member funtion of \"resetElementNumber\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
			errorMessage += "Message : \"elementSymbol\" .\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
			resetElementSymbol(elementSymbol);
	}
}

void AbstractChemicalElement::resetElementSymbol(const std::string& elementSymbol)
{
	validateElementSymbol(elementSymbol);

	m_isAnion = isAnion(elementSymbol);
	m_bondingAffinity = bondingAffinity(elementSymbol);
	m_coordinationNumber = coordinationNumber(elementSymbol);
	m_maxCommonBridgingAtomsSharing = maxCommonBridgingAtomsSharing(elementSymbol);
	m_possibleMaxCoordinationNumber = possibleMaxCoordinationNumber(elementSymbol);

	m_metallicRadius = getMetallicRadius(m_isAnion);
	{
		if (m_isAnion)
		{
			m_minimumIonicRadius = getAnionicRadius(elementSymbol);
			m_maximumIonicRadius = getAnionicRadius(elementSymbol);
		}

		else
		{
			m_minimumIonicRadius = getMinCationicRadius(elementSymbol);
			m_maximumIonicRadius = getMaxCationicRadius(elementSymbol);
		}
	}
	m_elementSymbol = elementSymbol;
}

// Public
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

double AbstractChemicalElement::getAnionicRadius(const std::string& elementSymbol) const
{
	if (elementSymbol == "O")
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.4);

	else if (elementSymbol == "S")
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.84);

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member funtion of \"getAnionicRadius\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
		errorMessage += "Message : \"elementSymbol\" is not a symbol for the anion.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

double AbstractChemicalElement::getMinCationicRadius(const std::string& elementSymbol) const
{
	if (elementSymbol == "Na")
		return getClosedPackedCationicRadius(12);

	else if (elementSymbol == "Al")
		return getClosedPackedCationicRadius(4);
	else if (elementSymbol == "Si")
		return getClosedPackedCationicRadius(4);
	else if (elementSymbol == "P")
		return getClosedPackedCationicRadius(4);

	else if (elementSymbol == "K")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Ca")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Sc")
		return getClosedPackedCationicRadius(12);

	else if (elementSymbol == "Ti")
		return getClosedPackedCationicRadius(6);

	else if (elementSymbol == "Fe")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Co")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Ni")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Cu")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Zn")
		return getClosedPackedCationicRadius(4);
	else if (elementSymbol == "Ga")
		return getClosedPackedCationicRadius(5);

	else if (elementSymbol == "Y")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Zr")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Nb")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Mo")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Tc")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Ru")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Rh")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Pd")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Ag")
		return getClosedPackedCationicRadius(6);

	else if (elementSymbol == "In")
		return getClosedPackedCationicRadius(7);

	else if (elementSymbol == "La")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Ce")
		return getClosedPackedCationicRadius(10);
	else if (elementSymbol == "Pr")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Nd")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Pm")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Sm")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Eu")
		return getClosedPackedCationicRadius(11);
	else if (elementSymbol == "Gd")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Tb")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Dy")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Ho")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Er")
		return getClosedPackedCationicRadius(10);
	else if (elementSymbol == "Tm")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Yb")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Lu")
		return getClosedPackedCationicRadius(6);

	else if (elementSymbol == "Pu")
		return getClosedPackedCationicRadius(12);

	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member funtion of \"getMinCationicRadius\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
		errorMessage += "Message : \"elementSymbol\" is out of definitions.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

double AbstractChemicalElement::getMaxCationicRadius(const std::string& elementSymbol) const
{
	if (elementSymbol == "Na")
		return getClosedPackedCationicRadius(12);

	else if (elementSymbol == "Al")
		return getClosedPackedCationicRadius(4);
	else if (elementSymbol == "Si")
		return getClosedPackedCationicRadius(4);
	else if (elementSymbol == "P")
		return getClosedPackedCationicRadius(6);

	else if (elementSymbol == "K")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Ca")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Sc")
		return getClosedPackedCationicRadius(12);

	else if (elementSymbol == "Ti")
		return getClosedPackedCationicRadius(6);

	else if (elementSymbol == "Fe")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Co")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Ni")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Cu")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Zn")
		return getClosedPackedCationicRadius(6);
	else if (elementSymbol == "Ga")
		return getClosedPackedCationicRadius(8);

	else if (elementSymbol == "Y")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Zr")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Nb")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Mo")
		return getClosedPackedCationicRadius(9);
	else if (elementSymbol == "Tc")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Ru")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Rh")
		return getClosedPackedCationicRadius(8);
	else if (elementSymbol == "Pd")
		return getClosedPackedCationicRadius(7);
	else if (elementSymbol == "Ag")
		return getClosedPackedCationicRadius(7);

	else if (elementSymbol == "In")
		return getClosedPackedCationicRadius(7);

	else if (elementSymbol == "La")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Ce")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Pr")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Nd")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Pm")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Sm")
		return getClosedPackedCationicRadius(12);
	else if (elementSymbol == "Eu")
		return getClosedPackedCationicRadius(11);
	else if (elementSymbol == "Gd")
		return getClosedPackedCationicRadius(11);
	else if (elementSymbol == "Tb")
		return getClosedPackedCationicRadius(11);
	else if (elementSymbol == "Dy")
		return getClosedPackedCationicRadius(11);
	else if (elementSymbol == "Ho")
		return getClosedPackedCationicRadius(11);
	else if (elementSymbol == "Er")
		return getClosedPackedCationicRadius(10);
	else if (elementSymbol == "Tm")
		return getClosedPackedCationicRadius(10);
	else if (elementSymbol == "Yb")
		return getClosedPackedCationicRadius(10);
	else if (elementSymbol == "Lu")
		return getClosedPackedCationicRadius(10);

	else if (elementSymbol == "Pu")
		return getClosedPackedCationicRadius(12);

	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member funtion of \"getMaxCationicRadius\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
		errorMessage += "Message : \"elementSymbol\" is out of definitions.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

double AbstractChemicalElement::getClosedPackedCationicRadius(const size_type possibleMaxCoordinationNumber) const
{
	switch (possibleMaxCoordinationNumber)
	{
	case 3:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.2165807537);
	case 4:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.3146428199);
	case 5:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.5798989873);
	case 6:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.5798989873);
	case 7:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.8277554212);
	case 8:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.9034602864);
	case 9:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.0248711306);
	case 10:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.165450385);
	case 11:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.2629582456);
	case 12:
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.4);
	}

	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member funtion of \"getClosedPackedCationicRadius\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
		errorMessage += "Message : \"possibleMaxCoordinationNumber\" is out of definitions.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

double AbstractChemicalElement::getMetallicRadius(const bool isAnion) const
{
	if (isAnion)
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(0.6);
	else
		return SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(1.4);
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private translators for element symbols

bool AbstractChemicalElement::isAnion(const std::string& elementSymbol) const
{
	validateElementSymbol(elementSymbol);

	if (elementSymbol == "O")
		return true;

	else if (elementSymbol == "S")
		return true;

	else
		return false;
}

double AbstractChemicalElement::bondingAffinity(const std::string& elementSymbol) const
{
	if (elementSymbol == "O")
		return 1.0;

	else if (elementSymbol == "S")
		return 0.9;


	else if (elementSymbol == "Na")
		return 4.0;

	else if (elementSymbol == "Al")
		return 5.0;
	else if (elementSymbol == "Si")
		return 5.0;
	else if (elementSymbol == "P")
		return 5.0;

	else if (elementSymbol == "K")
		return 4.0;
	else if (elementSymbol == "Ca")
		return 4.0;
	else if (elementSymbol == "Sc")
		return 4.0;

	else if (elementSymbol == "Ti")
		return 4.0;

	else if (elementSymbol == "Fe")
		return 4.5;
	else if (elementSymbol == "Co")
		return 4.5;
	else if (elementSymbol == "Ni")
		return 4.5;
	else if (elementSymbol == "Cu")
		return 1.5;
	else if (elementSymbol == "Zn")
		return 1.5;
	else if (elementSymbol == "Ga")
		return 1.2;

	else if (elementSymbol == "Y")
		return 1.5;
	else if (elementSymbol == "Zr")
		return 1.5;
	else if (elementSymbol == "Nb")
		return 1.5;
	else if (elementSymbol == "Mo")
		return 1.5;
	else if (elementSymbol == "Tc")
		return 1.5;
	else if (elementSymbol == "Ru")
		return 1.5;
	else if (elementSymbol == "Rh")
		return 1.5;
	else if (elementSymbol == "Pd")
		return 1.5;
	else if (elementSymbol == "Ag")
		return 1.5;

	else if (elementSymbol == "In")
		return 1.1;

	else if (elementSymbol == "La")
		return 1.1;
	else if (elementSymbol == "Ce")
		return 1.1;
	else if (elementSymbol == "Pr")
		return 1.1;
	else if (elementSymbol == "Nd")
		return 1.1;
	else if (elementSymbol == "Pm")
		return 1.1;
	else if (elementSymbol == "Sm")
		return 1.1;
	else if (elementSymbol == "Eu")
		return 1.1;
	else if (elementSymbol == "Gd")
		return 1.1;
	else if (elementSymbol == "Tb")
		return 1.1;
	else if (elementSymbol == "Dy")
		return 1.1;
	else if (elementSymbol == "Ho")
		return 1.1;
	else if (elementSymbol == "Er")
		return 1.1;
	else if (elementSymbol == "Tm")
		return 1.1;
	else if (elementSymbol == "Yb")
		return 1.1;
	else if (elementSymbol == "Lu")
		return 1.1;

	else if (elementSymbol == "Pu")
		return 0.0001;

	else
		return 0.0;
}

AbstractChemicalElement::size_type AbstractChemicalElement::coordinationNumber(const std::string& elementSymbol) const
{
	if (elementSymbol == "O")
		return std::numeric_limits<size_type>::max();

	else if (elementSymbol == "S")
		return std::numeric_limits<size_type>::max();

	else if (elementSymbol == "Na")
		return 8;

	else if (elementSymbol == "Al")
		return 4;
	else if (elementSymbol == "Si")
		return 4;
	else if (elementSymbol == "P")
		return 4;

	else if (elementSymbol == "K")
		return 11;
	else if (elementSymbol == "Ca")
		return 10;
	else if (elementSymbol == "Sc")
		return 9;

	else if (elementSymbol == "Ti")
		return 6;
	
	else if (elementSymbol == "Fe")
		return 6;
	else if (elementSymbol == "Co")
		return 5;
	else if (elementSymbol == "Ni")
		return 4;
	else if (elementSymbol == "Cu")
		return 4;
	else if (elementSymbol == "Zn")
		return 5;
	else if (elementSymbol == "Ga")
		return 5;

	else if (elementSymbol == "Y")
		return 9;
	else if (elementSymbol == "Zr")
		return 8;
	else if (elementSymbol == "Nb")
		return 7;
	else if (elementSymbol == "Mo")
		return 6;
	else if (elementSymbol == "Tc")
		return 8;
	else if (elementSymbol == "Ru")
		return 7;
	else if (elementSymbol == "Rh")
		return 6;
	else if (elementSymbol == "Pd")
		return 7;
	else if (elementSymbol == "Ag")
		return 6;

	else if (elementSymbol == "In")
		return 6;

	else if (elementSymbol == "La")
		return 12;
	else if (elementSymbol == "Ce")
		return 11;
	else if (elementSymbol == "Pr")
		return 10;
	else if (elementSymbol == "Nd")
		return 9;
	else if (elementSymbol == "Pm")
		return 8;
	else if (elementSymbol == "Sm")
		return 7;
	else if (elementSymbol == "Eu")
		return 11;
	else if (elementSymbol == "Gd")
		return 10;
	else if (elementSymbol == "Tb")
		return 9;
	else if (elementSymbol == "Dy")
		return 8;
	else if (elementSymbol == "Ho")
		return 7;
	else if (elementSymbol == "Er")
		return 10;
	else if (elementSymbol == "Tm")
		return 9;
	else if (elementSymbol == "Yb")
		return 8;
	else if (elementSymbol == "Lu")
		return 7;

	else if (elementSymbol == "Pu")
		return 10000;

	else
		return 0;
}

AbstractChemicalElement::size_type AbstractChemicalElement::maxCommonBridgingAtomsSharing(const std::string& elementSymbol) const
{
	if (elementSymbol == "O")
		return std::numeric_limits<size_type>::max();
	else if (elementSymbol == "S")
		return std::numeric_limits<size_type>::max();

	else if (elementSymbol == "Na")
		return 4;

	else if (elementSymbol == "Al")
		return 2;
	else if (elementSymbol == "Si")
		return 1;
	else if (elementSymbol == "P")
		return 1;

	else if (elementSymbol == "K")
		return 4;
	else if (elementSymbol == "Ca")
		return 4;
	else if (elementSymbol == "Sc")
		return 4;

	else if (elementSymbol == "Ti")
		return 2;

	else if (elementSymbol == "Fe")
		return 3;
	else if (elementSymbol == "Co")
		return 3;
	else if (elementSymbol == "Ni")
		return 1;
	else if (elementSymbol == "Cu")
		return 2;
	else if (elementSymbol == "Zn")
		return 2;
	else if (elementSymbol == "Ga")
		return 3;

	else if (elementSymbol == "Y")
		return 2;
	else if (elementSymbol == "Zr")
		return 4;
	else if (elementSymbol == "Nb")
		return 2;
	else if (elementSymbol == "Mo")
		return 2;
	else if (elementSymbol == "Tc")
		return 4;
	else if (elementSymbol == "Ru")
		return 4;
	else if (elementSymbol == "Rh")
		return 2;
	else if (elementSymbol == "Pd")
		return 4;
	else if (elementSymbol == "Ag")
		return 4;

	else if (elementSymbol == "In")
		return 2;

	else if (elementSymbol == "La")
		return 4;
	else if (elementSymbol == "Ce")
		return 4;
	else if (elementSymbol == "Pr")
		return 4;
	else if (elementSymbol == "Nd")
		return 4;
	else if (elementSymbol == "Pm")
		return 4;
	else if (elementSymbol == "Sm")
		return 4;
	else if (elementSymbol == "Eu")
		return 4;
	else if (elementSymbol == "Gd")
		return 4;
	else if (elementSymbol == "Tb")
		return 4;
	else if (elementSymbol == "Dy")
		return 4;
	else if (elementSymbol == "Ho")
		return 4;
	else if (elementSymbol == "Er")
		return 4;
	else if (elementSymbol == "Tm")
		return 4;
	else if (elementSymbol == "Yb")
		return 4;
	else if (elementSymbol == "Lu")
		return 4;

	else if (elementSymbol == "Pu")
		return 4;

	else
		return 0;
}

AbstractChemicalElement::size_type AbstractChemicalElement::possibleMaxCoordinationNumber(const std::string& elementSymbol) const
{
	if (elementSymbol == "O")
		return std::numeric_limits<size_type>::max();

	if (elementSymbol == "S")
		return std::numeric_limits<size_type>::max();

	else if (elementSymbol == "Na")
		return 8;

	else if (elementSymbol == "Al")
		return 4;
	else if (elementSymbol == "Si")
		return 4;
	else if (elementSymbol == "P")
		return 4;

	else if (elementSymbol == "K")
		return 12;
	else if (elementSymbol == "Ca")
		return 12;
	else if (elementSymbol == "Sc")
		return 12;

	else if (elementSymbol == "Ti")
		return 6;

	else if (elementSymbol == "Fe")
		return 6;
	else if (elementSymbol == "Co")
		return 5;
	else if (elementSymbol == "Ni")
		return 4;
	else if (elementSymbol == "Cu")
		return 4;
	else if (elementSymbol == "Zn")
		return 5;
	else if (elementSymbol == "Ga")
		return 5;

	else if (elementSymbol == "Y")
		return 9;
	else if (elementSymbol == "Zr")
		return 9;
	else if (elementSymbol == "Nb")
		return 9;
	else if (elementSymbol == "Mo")
		return 9;
	else if (elementSymbol == "Tc")
		return 8;
	else if (elementSymbol == "Ru")
		return 8;
	else if (elementSymbol == "Rh")
		return 8;
	else if (elementSymbol == "Pd")
		return 7;
	else if (elementSymbol == "Ag")
		return 7;

	else if (elementSymbol == "In")
		return 6;

	else if (elementSymbol == "La")
		return 12;
	else if (elementSymbol == "Ce")
		return 12;
	else if (elementSymbol == "Pr")
		return 12;
	else if (elementSymbol == "Nd")
		return 12;
	else if (elementSymbol == "Pm")
		return 12;
	else if (elementSymbol == "Sm")
		return 12;
	else if (elementSymbol == "Eu")
		return 11;
	else if (elementSymbol == "Gd")
		return 11;
	else if (elementSymbol == "Tb")
		return 11;
	else if (elementSymbol == "Dy")
		return 11;
	else if (elementSymbol == "Ho")
		return 11;
	else if (elementSymbol == "Er")
		return 10;
	else if (elementSymbol == "Tm")
		return 10;
	else if (elementSymbol == "Yb")
		return 10;
	else if (elementSymbol == "Lu")
		return 10;

	else if (elementSymbol == "Pu")
		return 12;

	else
		return 0;
}

void AbstractChemicalElement::validateElementSymbol(const std::string& elementSymbol) const
{
	if (getElementNumber(elementSymbol) == 0)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The member funtion of \"validateElementSymbol\" in the class of SolidStatePhysics::Crystallization::Elements::AbstractChemicalElement.\n";
		errorMessage += "Message : \"elementSymbol\" is out of definitions.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Private translators for element symbols
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
