#include "ChemicalBondingRecorder.h"

using namespace SolidStatePhysics::Crystallization::Simulators;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

ChemicalBondingRecorder::size_type ChemicalBondingRecorder::m_maxCashingCounts{ 10000 };


ChemicalBondingRecorder::ChemicalBondingRecorder()
	: FileOwner{}
	, m_currentTimeCounts{ 0 }
{
}

ChemicalBondingRecorder::ChemicalBondingRecorder(const std::filesystem::path& dynamicsReporterPath)
	: FileOwner{ dynamicsReporterPath }
	, m_currentTimeCounts{ 0 }
{
	if (!(isCorrectExtension(filePath())))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Crystallization::Monitors::DynamicsReporterOwner.\n";
		errorMessage += "Message : The extension of dynamicsReporterPath, which is ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ", is not \"";
			errorMessage += correctExtension();
			errorMessage += "\".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

ChemicalBondingRecorder::ChemicalBondingRecorder(std::filesystem::path&& dynamicsReporterPath)
	: FileOwner{ std::move(dynamicsReporterPath) }
	, m_currentTimeCounts{ 0 }
{
	if (!(isCorrectExtension(filePath())))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Crystallization::Monitors::DynamicsReporterOwner.\n";
		errorMessage += "Message : The extension of dynamicsReporterPath, which is ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ", is not \"";
			errorMessage += correctExtension();
			errorMessage += "\".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void ChemicalBondingRecorder::createPlotFile()
{
	std::string outputString;
	{
		outputString += "set terminal postscript enhanced eps color size 7,3\n";
		outputString += "set output \"";
		{
			outputString += filePath().stem().string();
			outputString += ".eps\"\n\n";
		}

		outputString += "set xrange [0:";
		{
			outputString += std::to_string(currentTimeCounts() + (currentTimeCounts() / 100));
			outputString += "]\n";
		}
		outputString += "set yrange [0.0:1.3]\n";

		outputString += "set bmargin 4\n";
		outputString += "set ylabel offset 0.5,0.5\n";
		outputString += "set tics font \"Times-Roman,18\"\n";
		outputString += "set xlabel \"The Number of geometrical optimization steps\" font \"Times-Roman,22\"\n";
		outputString += "set ylabel \"Rate\" font \"Times-Roman,22\"\n";
		outputString += "set key right top font \"Times-Roman,18\" spacing 1.1 maxcols 1 samplen 1\n\n";

		outputString += "plot \\\n";
		{
			outputString += "     \"";
			outputString += filePath().filename().string();
			outputString += "\" u 1:2 with line lc rgb \"red\" t \"Chemical Bonding :\", \\\n";

			outputString += "     \"";
			outputString += filePath().filename().string();
			outputString += "\" u 1:3 with line lc rgb \"blue\" t \"Relaxed Chemical Bonds :\", \\\n";

			outputString += "     \"";
			outputString += filePath().filename().string();
			outputString += "\" u 1:4 with line lc rgb \"green\" t \"Packing Fraction :\", \\\n";
		}
	}


	FileOwner plotterOwner;
	{
		std::filesystem::path plotterPath{ filePath().parent_path() };
		{
			std::string filename = filePath().stem().string();
			filename += ".gpl";
			plotterPath /= filename;
		}

		plotterOwner = FileOwner{ plotterPath };
	}

	plotterOwner.overCreateEmptyFile();
	plotterOwner.write(outputString);
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
