#include "ChemicalConstituents.h"

#include <vector>
#include <numeric>

#include "PeriodicTable.h"

using namespace SolidStatePhysics::Materials;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Utility

ChemicalComposition ChemicalConstituents::toChemicalComposition() const noexcept
{
	size_type greatestCommonDivisor = 0;
	{
		std::vector<int> compositionCounts;

		for (const auto& numAndCount : dictionary())
			compositionCounts.push_back(static_cast<int>(numAndCount.second));

		int gcd = std::accumulate(compositionCounts.begin(), compositionCounts.end(), 0, [](int a, int b) {return std::gcd(a, b); });
		greatestCommonDivisor = static_cast<size_type>(gcd);
	}


	std::map<size_type, size_type> chemicalCompositionDictionary;
	{
		for (const auto& numAndCount : dictionary())
		{
			size_type count = (numAndCount.second / greatestCommonDivisor);
			chemicalCompositionDictionary.emplace(numAndCount.first, count);
		}
	}

	return ChemicalComposition(std::move(chemicalCompositionDictionary));
}

// Utility
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
