#include "CifFileOwner.h"

#include <sstream>
#include <fstream>

#include "PeriodicTable.h"

using namespace SolidStatePhysics::Materials::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

CifFileOwner::CifFileOwner(const std::filesystem::path& cifFilePath)
	: FileOwner{ cifFilePath }
{
	if (!(isCorrectExtension(filePath())))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Materials::IO::CifFileOwner.\n";
		errorMessage += "Message : The extension of cifFilePath, which is ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ", is not \"";
			errorMessage += correctExtension();
			errorMessage += "\".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

CifFileOwner::CifFileOwner(std::filesystem::path&& cifFilePath)
	: FileOwner{ std::move(cifFilePath) } 
{
	if (!(isCorrectExtension(filePath())))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Materials::IO::CifFileOwner.\n";
		errorMessage += "Message : The extension of cifFilePath, which is ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ", is not \"";
			errorMessage += correctExtension();
			errorMessage += "\".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

std::string CifFileOwner::writeHeaderMessage(const std::string& materialName) const
{
	std::string headerString;
	headerString += "data_";
	{
		headerString += materialName;
		headerString += "\n";
	}
	headerString += "_symmetry_space_group_name_H-M    'P1'\n";

	return headerString;
}

std::string CifFileOwner::writeCrystalInfo(const ChemicalConstituents& chemicalConstituents, const double cellVolume) const
{
	ChemicalComposition chemicalComposition = chemicalConstituents.toChemicalComposition();

	std::string outputText;
	outputText += "_symmetry_Int_Tables_number   1\n";
	outputText += "_chemical_formula_structural   ";
	std::size_t firstElementCountStructural = 0;
	{
		for (const auto& numAndCounting : chemicalComposition)
		{
			outputText += SolidStatePhysics::ChemicalElements::PeriodicTable::replyElementSymbol(numAndCounting.first);
			outputText += std::to_string(numAndCounting.second);
		}

		outputText += "\n";
		firstElementCountStructural = chemicalComposition.begin()->second;
	}
	outputText += "_chemical_formula_sum   \'";

	std::size_t firstElementCountSum = 0;
	{
		for (const auto& numAndCounting : chemicalConstituents)
		{
			outputText += SolidStatePhysics::ChemicalElements::PeriodicTable::replyElementSymbol(numAndCounting.first);
			outputText += std::to_string(numAndCounting.second);
			outputText += " ";
		}

		outputText.pop_back();
		outputText += "\'\n";
		firstElementCountSum = chemicalConstituents.begin()->second;
	}
	outputText += "_cell_volume   ";
	{
		outputText += std::to_string(cellVolume);
		outputText += "\n";
	}
	outputText += "_cell_formula_units_Z   ";
	{
		outputText += std::to_string(firstElementCountSum / firstElementCountStructural);
		outputText += "\n";
	}
	outputText += "loop_\n";
	outputText += " _symmetry_equiv_pos_site_id\n";
	outputText += " _symmetry_equiv_pos_as_xyz\n";
	outputText += "  1  \'x, y, z\'\n";
	outputText += "loop_\n";

	return outputText;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
