#include "ConstraintOwners.h"

#include "PeriodicLiquid.h"

#include "ChemicalBond.h"
#include "IonicConstrainter.h"
#include "MetallicConstrainter.h"
#include "PolyhedralConstrainter.h"
#include "SecondNeighborConstrainter.h"

using namespace SolidStatePhysics::Crystallization::Components;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of ForceOwner

ConstraintOwner::ConstraintOwner(InteractingAtom& ownerAtom)
	: m_ownerAtom{ std::ref(ownerAtom) }
	, m_constrainterPtr{}
{
}

ConstraintOwner::ConstraintOwner(ConstraintOwner&& constraintOwner) noexcept
	: m_ownerAtom{ constraintOwner.m_ownerAtom }
	, m_constrainterPtr{ std::move(constraintOwner.m_constrainterPtr) }
{
	m_constrainterPtr.reset();
}

ConstraintOwner& ConstraintOwner::operator=(ConstraintOwner&& constraintOwner) noexcept
{
	constraintOwner.m_constrainterPtr.reset();

	m_ownerAtom = constraintOwner.m_ownerAtom;
	m_constrainterPtr.reset();

	return *this;
}

// The class of ForceOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of AccessConstraintOwner

ChemicalBondOwner::ChemicalBondOwner(InteractingAtom& ownerAtom)
	: ConstraintOwner{ ownerAtom }
{
}

const ChemicalBond& ChemicalBondOwner::owingChemicalBond() const
{
	if (constrainterPtr())
	{
		auto chemicalBondPtr = std::dynamic_pointer_cast<ChemicalBond>(constrainterPtr());
		return *chemicalBondPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingChemicalBond\" in the class of SolidStatePhysics::Crystallization::Components::ChemicalBondOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

ChemicalBond& ChemicalBondOwner::owingChemicalBond()
{
	if (constrainterPtr())
	{
		auto chemicalBondPtr = std::dynamic_pointer_cast<ChemicalBond>(constrainterPtr());
		return *chemicalBondPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingChemicalBond\" in the class of SolidStatePhysics::Crystallization::Components::ChemicalBondOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void ChemicalBondOwner::registerOwingConstrainter(PeriodicLiquid& periodicLiquid) const
{
	if (constrainterPtr())
	{
		auto chemicalBondPtr = std::dynamic_pointer_cast<ChemicalBond>(constrainterPtr());
		periodicLiquid.link(chemicalBondPtr);
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"registerOwingConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::ChemicalBondOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// The class of ChemicalBondOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of CovalentConstraintOwner

IonicConstraintOwner::IonicConstraintOwner(InteractingAtom& ownerAtom)
	: ConstraintOwner{ ownerAtom }
{
}

const IonicConstrainter& IonicConstraintOwner::owingIonicConstrainter() const
{
	if (constrainterPtr())
	{
		auto ionicConstrainterPtr = std::dynamic_pointer_cast<IonicConstrainter>(constrainterPtr());
		return *ionicConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingIonicConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::IonicConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

IonicConstrainter& IonicConstraintOwner::owingIonicConstrainter()
{
	if (constrainterPtr())
	{
		auto ionicConstrainterPtr = std::dynamic_pointer_cast<IonicConstrainter>(constrainterPtr());
		return *ionicConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingIonicConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::IonicConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void IonicConstraintOwner::registerOwingConstrainter(PeriodicLiquid& periodicLiquid) const
{
	if (constrainterPtr())
	{
		auto ionicConstrainterPtr = std::dynamic_pointer_cast<IonicConstrainter>(constrainterPtr());
		periodicLiquid.link(ionicConstrainterPtr);
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"registerOwingConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::IonicConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// The class of IonicConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of MetallicConstraintOwner

MetallicConstraintOwner::MetallicConstraintOwner(InteractingAtom& ownerAtom)
	: ConstraintOwner{ ownerAtom }
{
}

const MetallicConstrainter& MetallicConstraintOwner::owingMetallicConstrainter() const
{
	if (constrainterPtr())
	{
		auto metallicConstrainterPtr = std::dynamic_pointer_cast<MetallicConstrainter>(constrainterPtr());
		return *metallicConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingMetallicConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::MetallicConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

MetallicConstrainter& MetallicConstraintOwner::owingMetallicConstrainter()
{
	if (constrainterPtr())
	{
		auto metallicConstrainterPtr = std::dynamic_pointer_cast<MetallicConstrainter>(constrainterPtr());
		return *metallicConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingCovalentConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::MetallicConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void MetallicConstraintOwner::registerOwingConstrainter(PeriodicLiquid& periodicLiquid) const
{
	if (constrainterPtr())
	{
		auto metallicConstrainterPtr = std::dynamic_pointer_cast<MetallicConstrainter>(constrainterPtr());
		periodicLiquid.link(metallicConstrainterPtr);
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"registerOwingConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::MetallicConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// The class of MetallicConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of PolyhedralConstraintOwner

PolyhedralConstraintOwner::PolyhedralConstraintOwner(InteractingAtom& ownerAtom)
	: ConstraintOwner{ ownerAtom }
{
}

const PolyhedralConstrainter& PolyhedralConstraintOwner::owingPolyhedralConstrainter() const
{
	if (constrainterPtr())
	{
		auto polyhedralConstrainterPtr = std::dynamic_pointer_cast<PolyhedralConstrainter>(constrainterPtr());
		return *polyhedralConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingPolyhedralConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::PolyhedralConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

PolyhedralConstrainter& PolyhedralConstraintOwner::owingPolyhedralConstrainter()
{
	if (constrainterPtr())
	{
		auto polyhedralConstrainterPtr = std::dynamic_pointer_cast<PolyhedralConstrainter>(constrainterPtr());
		return *polyhedralConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingPolyhedralConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::PolyhedralConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void PolyhedralConstraintOwner::registerOwingConstrainter(PeriodicLiquid& periodicLiquid) const
{
	if (constrainterPtr())
	{
		auto polyhedralConstrainterPtr = std::dynamic_pointer_cast<PolyhedralConstrainter>(constrainterPtr());
		periodicLiquid.link(polyhedralConstrainterPtr);
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"registerOwingConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::PolyhedralConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// The class of PolyhedralConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// The class of SecondNeighborConstraintOwner

SecondNeighborConstraintOwner::SecondNeighborConstraintOwner(InteractingAtom& ownerAtom)
	: ConstraintOwner{ ownerAtom }
{
}

const SecondNeighborConstrainter& SecondNeighborConstraintOwner::owingSecondNeighborConstrainter() const
{
	if (constrainterPtr())
	{
		auto secondNeighborConstrainterPtr = std::dynamic_pointer_cast<SecondNeighborConstrainter>(constrainterPtr());
		return *secondNeighborConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingSecondNeighborConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::SecondNeighborConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

SecondNeighborConstrainter& SecondNeighborConstraintOwner::owingSecondNeighborConstrainter()
{
	if (constrainterPtr())
	{
		auto secondNeighborConstrainterPtr = std::dynamic_pointer_cast<SecondNeighborConstrainter>(constrainterPtr());
		return *secondNeighborConstrainterPtr;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"owingSecondNeighborConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::SecondNeighborConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

void SecondNeighborConstraintOwner::registerOwingConstrainter(PeriodicLiquid& periodicLiquid) const
{
	if (constrainterPtr())
	{
		auto secondNeighborConstrainterPtr = std::dynamic_pointer_cast<SecondNeighborConstrainter>(constrainterPtr());
		periodicLiquid.link(secondNeighborConstrainterPtr);
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Logic error\n";
		errorMessage += "Location: The member function of \"registerOwingConstrainter\" in the class of SolidStatePhysics::Crystallization::Components::SecondNeighborConstraintOwner.\n";
		errorMessage += "Message : This is inactive.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::logic_error(errorMessage);
	}
}

// The class of SecondNeighborConstraintOwner
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
