#include "DatFileOwner.h"

#include <fstream>

#include "Clock.h"
#include "PeriodicTable.h"

using namespace SolidStatePhysics::Materials::IO;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

DatFileOwner::DatFileOwner(const std::filesystem::path& datFilePath)
	: FileOwner{ datFilePath }
{
	if (!(isCorrectExtension(filePath())))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Materials::IO::DatFileOwner.\n";
		errorMessage += "Message : The extension of datFilePath, which is ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ", is not \"";
			errorMessage += correctExtension();
			errorMessage += "\".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

DatFileOwner::DatFileOwner(std::filesystem::path&& datFilePath)
	: FileOwner{ std::move(datFilePath) }
{
	if (!(isCorrectExtension(filePath())))
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of SolidStatePhysics::Materials::IO::DatFileOwner.\n";
		errorMessage += "Message : The extension of datFilePath, which is ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ", is not \"";
			errorMessage += correctExtension();
			errorMessage += "\".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods to create a new dat file

std::string DatFileOwner::writeHeaderMessage(const std::string& materialName) const
{
	std::string headerMessage;
	headerMessage += "----------------------------------------------------------------------------------------------------\n";
	headerMessage += "----------------------------------------------------------------------------------------------------\n\n";
	headerMessage += "Material name: ";
	{
		headerMessage += materialName;
		headerMessage += "\n";
	}
	headerMessage += "Creation date: ";
	{
		headerMessage += ControlCenter::System::Clock::tellNowTime();
		headerMessage += "\n";
	}
	headerMessage += "\n----------------------------------------------------------------------------------------------------\n";
	headerMessage += "----------------------------------------------------------------------------------------------------\n\n";

	return headerMessage;
}

// Private methods to create a new dat file
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods to read a dat file

std::string DatFileOwner::readDatFileString() const
{
	std::ifstream inputFileStream;
	inputFileStream.open(filePath(), std::ios_base::in);

	if (!inputFileStream)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readDatFileString\" in the class of SolidStatePhysics::Materials::IO::DatFileOwner.\n";
		errorMessage += "Message : Could not open the owning file of ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}

	else
	{
		std::string inputString;
		{
			for (std::string stringLine; std::getline(inputFileStream, stringLine);)
			{
				inputString += stringLine;
				inputString += "\n";
			}

			inputFileStream.close();
		}

		return inputString;
	}
}

std::string DatFileOwner::readPrimitiveLatticeVectorsUnit(const std::string& inputString) const
{
	std::string latticeUnitType;
	{
		std::stringstream inputStream{ inputString };

		std::regex valueTypePat("[\\s]*Atoms.UnitVectors.Unit[\\s]+([\\w]+)[\\s]*");
		std::smatch valueTypeSm;

		for (std::string stringLine; std::getline(inputStream, stringLine);)
		{
			if (std::regex_match(stringLine, valueTypeSm, valueTypePat))
			{
				latticeUnitType = std::string{ valueTypeSm.str(1) };
				break;
			}
		}
	}

	if (latticeUnitType != "Ang" && latticeUnitType != "AU")
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readPrimitiveLatticeVectorsUnit\" in the class of SolidStatePhysics::Materials::IO::DatFileOwner.\n";
		errorMessage += "Message : Couldn't get Atoms.UnitVectors.Unit from the dat file of ";
		{
			errorMessage += filePath().generic_string();
			errorMessage += ".\n\n";
		}
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}

	else
		return latticeUnitType;
}

std::string DatFileOwner::readAtomicCoordinatesUnit(const std::string& inputString) const
{
	std::string coordinateUnitType;
	{
		std::stringstream inputStream{ inputString };

		std::regex valueTypePat("[\\s]*Atoms.SpeciesAndCoordinates.Unit[\\s]+([\\w]+)[\\s]*");
		std::smatch valueTypeSm;

		for (std::string stringLine; std::getline(inputStream, stringLine);)
		{
			if (std::regex_match(stringLine, valueTypeSm, valueTypePat))
			{
				coordinateUnitType = std::string{ valueTypeSm.str(1) };
				break;
			}
		}
	}

	if (coordinateUnitType == "FRAC" || coordinateUnitType == "Frac" || coordinateUnitType == "frac")
		return std::string{ "FRAC" };

	else
	{
		if (coordinateUnitType != "Ang" && coordinateUnitType != "AU")
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readAtomicCoordinatesUnit\" in the class of SolidStatePhysics::Materials::IO::DatFileOwner.\n";
			errorMessage += "Message : Atoms.SpeciesAndCoordinates.Unit in the dat file of ";
			{
				errorMessage += filePath().generic_string();
				errorMessage += " is not \"FRAC\", \"Frac\", \"frac\", \"Ang\" or \"AU\".\n\n";
			}
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}

		else
			return coordinateUnitType;
	}
}

DatFileOwner::CrystalLattice DatFileOwner::readPrimitiveLatticeVectors(const std::string& inputString) const
{
	NumericMatrix primitiveLatticeVectors;
	{
		std::stringstream inputStream{ inputString };

		for (std::string stringLine; std::getline(inputStream, stringLine);)
		{
			if (stringLine == "<Atoms.UnitVectors")
				break;
		}

		bool existFinalFlag = false;
		{
			for (std::string stringLine; std::getline(inputStream, stringLine);)
			{
				std::regex primitiveLatticeVectorsPat("[\\s]*([e\\d\\.\\-]+)[\\s]+([e\\d\\.\\-]+)[\\s]+([e\\d\\.\\-]+)[\\s]*");
				std::smatch NumericVectorSm;

				if (std::regex_match(stringLine, NumericVectorSm, primitiveLatticeVectorsPat))
				{
					NumericVector primitiveLatticeVector(3);
					std::stringstream latticeVectorStreamA{ NumericVectorSm.str(1) };
					std::stringstream latticeVectorStreamB{ NumericVectorSm.str(2) };
					std::stringstream latticeVectorStreamC{ NumericVectorSm.str(3) };
					latticeVectorStreamA >> primitiveLatticeVector[0];
					latticeVectorStreamB >> primitiveLatticeVector[1];
					latticeVectorStreamC >> primitiveLatticeVector[2];

					primitiveLatticeVectors.pushbackColumn(primitiveLatticeVector);
				}

				else
				{
					if (stringLine == "Atoms.UnitVectors>")
					{
						existFinalFlag = true;
						break;
					}
				}
			}

			if (!existFinalFlag || primitiveLatticeVectors.rowSize() != 3)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"readPrimitiveLatticeVectors\" in the class of SolidStatePhysics::Materials::IO::DatFileOwner.\n";
				errorMessage += "Message : Couldn't read the primitive lattice vectors from the dat file of ";
				{
					errorMessage += filePath().generic_string();
					errorMessage += ".\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}
		}
	}

	if (readPrimitiveLatticeVectorsUnit(inputString) == "Ang")
		primitiveLatticeVectors *= SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}.scalingMultiplier();

	return CrystalLattice{ std::move(primitiveLatticeVectors) };
}

// Private methods to read a dat file
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
