#include "DynamicsController.h"

#include <random>
#include <cmath>

using namespace SolidStatePhysics::Crystallization::Calculators;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

DynamicsController::DynamicsController()
	: m_maxDisplacement{ 0.0 }
	, m_coolingRate{ 0.0 }
	, m_maxLatticeTransformation{ 0.0 }
{
}

DynamicsController::DynamicsController(const double maxDisplacement, const double coolingRate)
	: m_maxDisplacement{ maxDisplacement }
	, m_coolingRate{ coolingRate }
	, m_maxLatticeTransformation{ 0.0 }
{
	{
		if (m_maxDisplacement < 0.0)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The constructor of the class of SolidStatePhysics::Crystallization::Calculators::DynamicsController.\n";
			errorMessage += "Message : \"m_maxDisplacement\" is less than zero.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}
	}

	{
		if (m_coolingRate < 0.0)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The constructor of the class of SolidStatePhysics::Crystallization::Calculators::DynamicsController.\n";
			errorMessage += "Message : \"m_coolingRate\" is less than zero.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
		{
			if (1.0 < m_coolingRate)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The constructor of the class of SolidStatePhysics::Crystallization::Calculators::DynamicsController.\n";
				errorMessage += "Message : \"m_coolingRate\" is more than one.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}
	}
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void DynamicsController::initialize() noexcept
{
	m_maxDisplacement = 0.0;
	m_coolingRate = 0.0;
	m_maxLatticeTransformation = 0.0;
}

void DynamicsController::clear() noexcept
{
	m_maxDisplacement = 0.0;
	m_coolingRate = 0.0;
	m_maxLatticeTransformation = 0.0;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
