#include "FinalSimulationsReporter.h"

#include "FileOwner.h"

using namespace SolidStatePhysics::Crystallization::Simulators;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

FinalSimulationsReporter::size_type FinalSimulationsReporter::m_elapsedTimesResolution{ 50 };


FinalSimulationsReporter::FinalSimulationsReporter()
	: m_workDirectoryPath{}
	, m_elapsedTimesDistance{ 1 }
	, m_elapsedTimes{}
	, m_localOptimizationHistories{}
	, m_remodellingHistories{}
{
}

FinalSimulationsReporter::FinalSimulationsReporter(const std::filesystem::path& workDirectoryPath, const size_type necessaryGeometricalOptimizing)
	: m_workDirectoryPath{ workDirectoryPath }
	, m_elapsedTimesDistance{ (necessaryGeometricalOptimizing / m_elapsedTimesResolution) }
	, m_elapsedTimes{}
	, m_localOptimizationHistories{}
	, m_remodellingHistories{}
{
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void FinalSimulationsReporter::initialize() noexcept
{
	m_elapsedTimes.clear();
	m_localOptimizationHistories.clear();
	m_remodellingHistories.clear();
}

void FinalSimulationsReporter::clear() noexcept
{
	m_elapsedTimes.clear();
	m_localOptimizationHistories.clear();
	m_remodellingHistories.clear();
}

void FinalSimulationsReporter::createFinalReportFile(const std::string reportString) const
{
	std::filesystem::path finalReportFilePath{ m_workDirectoryPath };
	finalReportFilePath /= "finalReport.txt";

	ControlCenter::System::IO::FileOwner reporterOwner{ finalReportFilePath };
	reporterOwner.overCreateEmptyFile();
	reporterOwner.write(reportString);
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void FinalSimulationsReporter::createElapsedTimesReportFile() const
{
	std::string reportString;
	{
		for (size_type index = 0; index < m_elapsedTimes.size(); ++index)
		{
			reportString += std::to_string(m_elapsedTimesDistance * (index + 1));
			reportString += "   ";
			reportString += std::to_string(m_elapsedTimes.at(index));
			reportString += "\n";
		}
	}

	ControlCenter::System::IO::FileOwner reporterOwner{ getElapsedTimesReportFilePath() };
	reporterOwner.overCreateEmptyFile();
	reporterOwner.write(reportString);
}

void FinalSimulationsReporter::createLocalOptimizationHistoriesReportFile() const
{
	std::string reportString;
	{
		size_type currentLocalOptimizing = 0;

		for (const auto& numAndCount : m_localOptimizationHistories)
		{
			for (; currentLocalOptimizing < numAndCount.first; ++currentLocalOptimizing)
			{
				reportString += std::to_string(currentLocalOptimizing);
				reportString += "   ";
				reportString += "0\n";
			}

			reportString += std::to_string(currentLocalOptimizing);
			reportString += "   ";
			reportString += std::to_string(numAndCount.second);
			reportString += "\n";
			++currentLocalOptimizing;
		}
	}

	ControlCenter::System::IO::FileOwner reporterOwner{ getLocalOptimizationHistoriesReportFilePath() };
	reporterOwner.overCreateEmptyFile();
	reporterOwner.write(reportString);
}

void FinalSimulationsReporter::createRemodellingHistoriesReportFile() const
{
	std::string reportString;
	{
		size_type currentRemodelling = 0;

		for (const auto& numAndCount : m_remodellingHistories)
		{
			for (; currentRemodelling < numAndCount.first; ++currentRemodelling)
			{
				reportString += std::to_string(currentRemodelling);
				reportString += "   ";
				reportString += "0\n";
			}

			reportString += std::to_string(currentRemodelling);
			reportString += "   ";
			reportString += std::to_string(numAndCount.second);
			reportString += "\n";
			++currentRemodelling;
		}
	}

	ControlCenter::System::IO::FileOwner reporterOwner{ getRemodellingHistoriesReportFilePath() };
	reporterOwner.overCreateEmptyFile();
	reporterOwner.write(reportString);
}

void FinalSimulationsReporter::createElapsedTimesPlotFile() const
{
	std::string outputString;
	{
		outputString += "set terminal postscript enhanced eps color size 7,3\n";
		outputString += "set output \"";
		{
			outputString += getElapsedTimesPlotFilePath().stem().string();
			outputString += ".eps\"\n\n";
		}

		outputString += "set xrange [0:";
		{
			outputString += std::to_string(getMaxElapsedTime() + m_elapsedTimesDistance);
			outputString += "]\n";
		}
		outputString += "set yrange [0:";
		{
			outputString += std::to_string(getMaxSampleCountInElapsedTimes() + (getMaxSampleCountInElapsedTimes() / 10));
			outputString += "]\n\n";
		}

		outputString += "set bmargin 4\n";
		outputString += "set ylabel offset 0.5,0.5\n";
		outputString += "set tics font \"Times-Roman,18\"\n";
		outputString += "set xlabel \"Elapsed time\" font \"Times-Roman,22\"\n";
		outputString += "set ylabel \"Number of samples\" font \"Times-Roman,22\"\n";
		outputString += "set key outside font \"Times-Roman,18\" spacing 1.1 maxcols 1 samplen 0\n";
		outputString += "unset key\n\n";

		outputString += "plot ";
		{
			outputString += "\"";
			outputString += getElapsedTimesReportFilePath().filename().string();
			outputString += "\" u 1:2 with boxes\n";
		}
	}


	ControlCenter::System::IO::FileOwner plotterOwner{ getElapsedTimesPlotFilePath() };
	plotterOwner.overCreateEmptyFile();
	plotterOwner.write(outputString);
}

void FinalSimulationsReporter::createLocalOptimizationHistoriesPlotFile() const
{
	std::string outputString;
	{
		outputString += "set terminal postscript enhanced eps color size 7,3\n";
		outputString += "set output \"";
		{
			outputString += getLocalOptimizationHistoriesPlotFilePath().stem().string();
			outputString += ".eps\"\n\n";
		}

		outputString += "set xrange [0:";
		{
			outputString += std::to_string(getMaxLocalOptimizing() + (getMaxLocalOptimizing() / 10));
			outputString += "]\n";
		}
		outputString += "set yrange [0:";
		{
			outputString += std::to_string(getMaxSampleCountInLocalOptimizationHistories() + (getMaxSampleCountInLocalOptimizationHistories() / 10));
			outputString += "]\n\n";
		}

		outputString += "set bmargin 4\n";
		outputString += "set ylabel offset 0.5,0.5\n";
		outputString += "set tics font \"Times-Roman,18\"\n";
		outputString += "set xlabel \"Local optimization counts\" font \"Times-Roman,22\"\n";
		outputString += "set ylabel \"Number of samples\" font \"Times-Roman,22\"\n";
		outputString += "set key outside font \"Times-Roman,18\" spacing 1.1 maxcols 1 samplen 0\n";
		outputString += "unset key\n\n";

		outputString += "plot ";
		{
			outputString += "\"";
			outputString += getLocalOptimizationHistoriesReportFilePath().filename().string();
			outputString += "\" u 1:2 with boxes\n";
		}
	}


	ControlCenter::System::IO::FileOwner plotterOwner{ getLocalOptimizationHistoriesPlotFilePath() };
	plotterOwner.overCreateEmptyFile();
	plotterOwner.write(outputString);
}

void FinalSimulationsReporter::createRemodellingHistoriesPlotFile() const
{
	std::string outputString;
	{
		outputString += "set terminal postscript enhanced eps color size 7,3\n";
		outputString += "set output \"";
		{
			outputString += getRemodellingHistoriesPlotFilePath().stem().string();
			outputString += ".eps\"\n\n";
		}

		outputString += "set xrange [0:";
		{
			outputString += std::to_string(getMaxRemodelling() + (getMaxRemodelling() / 10));
			outputString += "]\n";
		}
		outputString += "set yrange [0:";
		{
			outputString += std::to_string(getMaxSampleCountInRemodellingHistories() + (getMaxSampleCountInRemodellingHistories() / 10));
			outputString += "]\n\n";
		}

		outputString += "set bmargin 4\n";
		outputString += "set ylabel offset 0.5,0.5\n";
		outputString += "set tics font \"Times-Roman,18\"\n";
		outputString += "set xlabel \"Remodelling counts\" font \"Times-Roman,22\"\n";
		outputString += "set ylabel \"Number of samples\" font \"Times-Roman,22\"\n";
		outputString += "set key outside font \"Times-Roman,18\" spacing 1.1 maxcols 1 samplen 0\n";
		outputString += "unset key\n\n";

		outputString += "plot ";
		{
			outputString += "\"";
			outputString += getRemodellingHistoriesReportFilePath().filename().string();
			outputString += "\" u 1:2 with boxes\n";
		}
	}


	ControlCenter::System::IO::FileOwner plotterOwner{ getRemodellingHistoriesPlotFilePath() };
	plotterOwner.overCreateEmptyFile();
	plotterOwner.write(outputString);
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
