#include "IntelligentAtom.h"

#include <limits>

#include "AbstractChemicalElement.h"

using namespace SolidStatePhysics::Crystallization::Components;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors and operators

IntelligentAtom::IntelligentAtom(const size_type atomicNumber)
	: InteractingAtom{ atomicNumber }
	, m_isAnion{ false }
	, m_maxCommonBridgingAtomsSharing{ 0 }
	, m_bondingMotivation{}
{
	SolidStatePhysics::Crystallization::Calculators::AbstractChemicalElement abstractChemicalElement;
	abstractChemicalElement.resetElementNumber(atomicNumber);
	m_isAnion = abstractChemicalElement.isAnion();
	m_maxCommonBridgingAtomsSharing = abstractChemicalElement.maxCommonBridgingAtomsSharing();
	m_bondingMotivation = abstractChemicalElement.getBondingMotivation();
}

IntelligentAtom::IntelligentAtom(const size_type atomicNumber, const NumericVector& cartesianCoordinate)
	: InteractingAtom{ atomicNumber, cartesianCoordinate }
	, m_isAnion{ false }
	, m_maxCommonBridgingAtomsSharing{ 0 }
	, m_bondingMotivation{}
{
	SolidStatePhysics::Crystallization::Calculators::AbstractChemicalElement abstractChemicalElement;
	abstractChemicalElement.resetElementNumber(atomicNumber);
	m_isAnion = abstractChemicalElement.isAnion();
	m_maxCommonBridgingAtomsSharing = abstractChemicalElement.maxCommonBridgingAtomsSharing();
	m_bondingMotivation = abstractChemicalElement.getBondingMotivation();
}

IntelligentAtom::IntelligentAtom(const IntelligentAtom& intelligentAtom)
	: InteractingAtom{ intelligentAtom }
	, m_isAnion{ intelligentAtom.m_isAnion }
	, m_maxCommonBridgingAtomsSharing{ intelligentAtom.m_maxCommonBridgingAtomsSharing }
	, m_bondingMotivation{ intelligentAtom.m_bondingMotivation }
{
}

IntelligentAtom::IntelligentAtom(IntelligentAtom&& intelligentAtom) noexcept
	: InteractingAtom{ std::move(intelligentAtom) }
	, m_isAnion{ intelligentAtom.m_isAnion }
	, m_maxCommonBridgingAtomsSharing{ intelligentAtom.m_maxCommonBridgingAtomsSharing }
	, m_bondingMotivation{ intelligentAtom.m_bondingMotivation }
{
	intelligentAtom.initialize();
}

IntelligentAtom& IntelligentAtom::operator=(const IntelligentAtom& intelligentAtom)
{
	InteractingAtom::operator=(intelligentAtom);
	IntelligentAtom::initialize();

	return *this;
}

IntelligentAtom& IntelligentAtom::operator=(IntelligentAtom&& intelligentAtom) noexcept
{
	InteractingAtom::operator=(std::move(intelligentAtom));
	IntelligentAtom::initialize();

	intelligentAtom.initialize();

	return *this;
}

// Constructors and operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void IntelligentAtom::initialize() noexcept
{
	InteractingAtom::initialize();
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
