#include "InteratomicConnector.h"

using namespace SolidStatePhysics::Crystallization::Calculators;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Protected constructors

InteratomicConnector::InteratomicConnector(InteractingAtom& ownerAtom, AtomicAdaptor&& attachedAtom)
	: m_crystalLattice{ std::ref(attachedAtom.crystalLattice()) }
	, m_ownerAtom{ std::ref(ownerAtom) }
	, m_attachedAtom{ std::move(attachedAtom) }
	, m_twinPtr{}
	, m_displacement{}
	, m_interatomicDistance{ 0.0 }
{
	if (m_attachedAtom.isInOriginalCell())
	{
		m_displacement = (m_attachedAtom.entity().cartesianCoordinate() - m_ownerAtom.get().cartesianCoordinate());
		m_interatomicDistance = m_displacement.norm();
	}

	else
	{
		m_displacement = (m_attachedAtom.entity().cartesianCoordinate() + m_attachedAtom.translationVector());
		m_displacement -= m_ownerAtom.get().cartesianCoordinate();
		m_interatomicDistance = m_displacement.norm();
	}
}

InteratomicConnector::InteratomicConnector(const std::shared_ptr<InteratomicConnector>& twinPtr)
	: m_crystalLattice{ twinPtr->m_crystalLattice }
	, m_ownerAtom{ std::ref(twinPtr->m_attachedAtom.entity()) }
	, m_attachedAtom{ twinPtr->m_ownerAtom, twinPtr->m_attachedAtom }
	, m_twinPtr{ twinPtr }
	, m_displacement{ (-1.0 * twinPtr->m_displacement) }
	, m_interatomicDistance{ twinPtr->m_interatomicDistance }
{
	m_attachedAtom.reverseLatticePoint();
}

// Protected constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
