#include "NumericMatrix.h"

#include <algorithm>

#include "NumericVector.h"
#include "NumericVectorSlice.h"

using namespace NumericalAnalysis::LinearAlgebra;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

NumericMatrix::NumericMatrix(std::initializer_list<NumericVector> init)
	: m_numericMatrix{}
	, m_columnSize{}
	, m_rowSize{ init.size() }
{
	m_columnSize = init.begin()->size();
	{
		for (auto& numericVector : init)
		{
			if (m_columnSize != numericVector.size())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : The sizes of NumericVector in initializer_list are different each other.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}

		if (m_columnSize == 0 || m_rowSize == 0)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
			errorMessage += "Message : The size of NumericVector in initializer_list and/or the size of initialize_list are zero.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}
	}

	m_numericMatrix.resize(m_columnSize * m_rowSize);
	{
		auto iter = std::begin(m_numericMatrix);

		for (auto& numericVector : init)
		{
			cblas_dcopy(m_columnSize, numericVector.begin(), 1, iter, 1);
			iter += m_columnSize;
		}
	}
}

NumericMatrix::NumericMatrix(const std::vector<NumericVector>& init)
	: m_numericMatrix{}
	, m_columnSize{}
	, m_rowSize{ init.size() }
{
	m_columnSize = init[0].size();
	{
		for (auto& numericVector : init)
		{
			if (m_columnSize != numericVector.size())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : The sizes of NumericVector in std::vector<NumericVector> are different each other.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}

		if (m_columnSize == 0 || m_rowSize == 0)
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
			errorMessage += "Message : The size of NumericVector in std::vector<NumericVector> and/or the size of std::vector<NumericVector> are zero.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}
	}

	m_numericMatrix.resize(m_columnSize * m_rowSize);
	{
		auto iter = std::begin(m_numericMatrix);

		for (auto& numericVector : init)
		{
			cblas_dcopy(m_columnSize, numericVector.begin(), 1, iter, 1);
			iter += m_columnSize;
		}
	}
}

NumericMatrix::NumericMatrix(const size_type columnSize, const size_type rowSize)
	: m_numericMatrix(columnSize * rowSize)
	, m_columnSize{ columnSize }
	, m_rowSize{ rowSize }
{
	if (m_columnSize == 0 || m_rowSize == 0)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : columnSize and/or rowSize are zero. --> columnSize and rowSize must be more than zero.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

NumericMatrix::NumericMatrix(const value_type value, const size_type columnSize, const size_type rowSize)
	: m_numericMatrix(value, columnSize * rowSize)
	, m_columnSize{ columnSize }
	, m_rowSize{ rowSize }
{
	if (m_columnSize == 0 || m_rowSize == 0)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : columnSize and/or rowSize are zero. --> columnSize and rowSize must be more than zero.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

NumericMatrix::NumericMatrix(const NumericMatrix& numericMatrix)
	: m_numericMatrix(numericMatrix.size())
	, m_columnSize{ numericMatrix.m_columnSize }
	, m_rowSize{ numericMatrix.m_rowSize }
{
	cblas_dcopy(m_numericMatrix.size(), numericMatrix.begin(), 1, std::begin(m_numericMatrix), 1);
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Arithmetic functions

NumericMatrix& NumericMatrix::add(const value_type scalingMultiplier, const NumericVector& leftNumericVector, const NumericVector& rightNumericVector) noexcept
{
	cblas_dger(CblasColMajor, m_columnSize, m_rowSize, scalingMultiplier, leftNumericVector.begin(), 1, rightNumericVector.begin(), 1, std::begin(m_numericMatrix), m_columnSize);
	return *this;
}

// Arithmetic functions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Slice functions

NumericVector NumericMatrix::operator[](const size_type columnIndex) const noexcept
{
	NumericVector result(m_columnSize);
	const_pointer firstIndex = std::begin(m_numericMatrix) + (columnIndex * m_columnSize);
	cblas_dcopy(m_columnSize, firstIndex, 1, result.begin(), 1);
	return result;
}

NumericVectorSlice NumericMatrix::operator[](const size_type columnIndex) noexcept
{
	return NumericVectorSlice(m_numericMatrix, std::slice(columnIndex * m_columnSize, m_columnSize, 1));
}

NumericVector NumericMatrix::columnVector(const size_type columnIndex) const
{
	if (columnIndex < m_rowSize)
	{
		NumericVector result(m_columnSize);
		const_pointer firstIndex = std::begin(m_numericMatrix) + (columnIndex * m_columnSize);
		cblas_dcopy(m_columnSize, firstIndex, 1, result.begin(), 1);
		return result;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"columnVector\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : columnIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}
}

NumericVectorSlice NumericMatrix::columnVectorSlice(const size_type columnIndex)
{
	if (columnIndex < m_rowSize)
	{
		return NumericVectorSlice(m_numericMatrix, std::slice(columnIndex * m_columnSize, m_columnSize, 1));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"columnVectorSlice\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : columnIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}
}

NumericVector NumericMatrix::rowVector(const size_type rowIndex) const
{
	if (rowIndex < m_columnSize)
	{
		NumericVector result(m_rowSize);
		const_pointer firstIndex = std::begin(m_numericMatrix) + rowIndex;
		cblas_dcopy(m_rowSize, firstIndex, m_columnSize, result.begin(), 1);
		return result;
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"rowVector\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : rowIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}
}

NumericVectorSlice NumericMatrix::rowVectorSlice(const size_type rowIndex)
{
	if (rowIndex < m_columnSize)
	{
		return NumericVectorSlice(m_numericMatrix, std::slice(rowIndex, m_rowSize, m_columnSize));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"rowVectorSlice\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : rowIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}
}

NumericVector NumericMatrix::replicateAsNumericVector() const noexcept
{
	NumericVector numericVector(m_numericMatrix.size());
	cblas_dcopy(numericVector.size(), std::begin(m_numericMatrix), 1, numericVector.begin(), 1);
	return numericVector;
}

NumericVector NumericMatrix::migrateToNumericVector() noexcept
{
	NumericVector numericVector;
	m_numericMatrix.swap(numericVector.m_numericVector);
	m_columnSize = 0;
	m_rowSize = 0;
	return numericVector;
}

NumericVectorSlice NumericMatrix::asNumericVectorSlice() noexcept
{
	return NumericVectorSlice(m_numericMatrix, std::slice(0, (m_columnSize * m_rowSize), 1));
}

// Slice functions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Customize functions

void NumericMatrix::insertColumn(const NumericVector& numericColumnVector, const size_type insertedColumnIndex)
{
	if (m_rowSize < insertedColumnIndex)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"insertColumn\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : insertedColumnIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}

	else
	{
		if (empty())
		{
			m_columnSize = numericColumnVector.size();
		}

		else
		{
			if (m_columnSize != numericColumnVector.size())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The function of \"insertColumn\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : The size of numericColumnVector do not correspond to the size of m_columnSize.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}
	}

	insertColumn(insertedColumnIndex, numericColumnVector.m_numericVector, 1);
}

void NumericMatrix::insertColumns(const NumericMatrix& numericMatrix, const size_type insertedColumnIndex)
{
	if (m_rowSize < insertedColumnIndex)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"insertColumns\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : insertedColumnIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}

	else
	{
		if (empty())
		{
			m_columnSize = numericMatrix.m_columnSize;
		}

		else
		{
			if (m_columnSize != numericMatrix.m_columnSize)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The function of \"insertColumns\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : The column size of numericMatrix do not correspond to the size of m_columnSize.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}
	}

	insertColumn(insertedColumnIndex, numericMatrix.m_numericMatrix, numericMatrix.m_rowSize);
}

void NumericMatrix::insertRow(const NumericVector& numericRowVector, const size_type insertedRowIndex)
{
	if (m_columnSize < insertedRowIndex)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"insertRow\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : insertedRowIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}

	else
	{
		if (empty())
		{
			m_rowSize = numericRowVector.size();
		}

		else
		{
			if (m_rowSize != numericRowVector.size())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The function of \"insertRow\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : The size of numericRowVector do not correspond to the size of m_rowSize.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}
	}

	insertRow(insertedRowIndex, numericRowVector.m_numericVector, 1);
}

void NumericMatrix::insertRows(const NumericMatrix& numericMatrix, const size_type insertedRowIndex)
{
	if (m_columnSize < insertedRowIndex)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Out of range\n";
		errorMessage += "Location: The function of \"insertRows\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
		errorMessage += "Message : insertedRowIndex is out of range.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::out_of_range(errorMessage);
	}

	else
	{
		if (empty())
		{
			m_rowSize = numericMatrix.m_rowSize;
		}

		else
		{
			if (m_rowSize != numericMatrix.m_rowSize)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Invalid argument\n";
				errorMessage += "Location: The function of \"insertRows\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : The row size of numericMatrix do not correspond to the size of m_rowSize.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::invalid_argument(errorMessage);
			}
		}
	}

	insertRow(insertedRowIndex, numericMatrix.m_numericMatrix, numericMatrix.m_columnSize);
}

void NumericalAnalysis::LinearAlgebra::NumericMatrix::eraseColumns(std::initializer_list<size_type> columnIndices)
{
	std::vector<size_type> sortedRemovedColumnIndices;
	{
		std::vector<size_type> originalRemovedColumnIndices{ columnIndices };
		std::sort(originalRemovedColumnIndices.begin(), originalRemovedColumnIndices.end());
		std::unique_copy(originalRemovedColumnIndices.begin(), originalRemovedColumnIndices.end(), std::back_inserter(sortedRemovedColumnIndices));

		if (sortedRemovedColumnIndices.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The function of \"removeColumn\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
			errorMessage += "Message : std::initializer_list<size_type> is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
		{
			if (m_rowSize <= sortedRemovedColumnIndices.size())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Logic error\n";
				errorMessage += "Location: The function of \"removeColumn\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : It is impossible to execute this function since m_rowSize is not more than the size of sortedRemovedColumnIndices.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::logic_error(errorMessage);
			}

			if (m_rowSize <= sortedRemovedColumnIndices.back())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Out of range\n";
				errorMessage += "Location: The function of \"removeColumn\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : removedColumnIndices contains an index being out of range.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::out_of_range(errorMessage);
			}
		}
	}

	const size_type removedColumnSize = sortedRemovedColumnIndices.size();
	std::valarray<value_type> renewedNumericMatrix(m_columnSize * (m_rowSize - removedColumnSize));

	pointer renewedNumericMatrixPointer = std::begin(renewedNumericMatrix);
	const_pointer numericMatrixPointer = std::begin(m_numericMatrix);

	sortedRemovedColumnIndices.push_back(0);
	std::vector<size_type>::const_iterator sortedRemovedIndicesIter = sortedRemovedColumnIndices.begin();

	for (size_type columnIndex = 0; columnIndex < m_rowSize; ++columnIndex)
	{
		if (columnIndex == *sortedRemovedIndicesIter)
		{
			numericMatrixPointer += m_columnSize;
			++sortedRemovedIndicesIter;
		}

		else
		{
			for (size_type rowIndex = 0; rowIndex < m_columnSize; ++rowIndex)
			{
				cblas_dcopy(m_columnSize, numericMatrixPointer, 1, renewedNumericMatrixPointer, 1);
				renewedNumericMatrixPointer += m_columnSize;
				numericMatrixPointer += m_columnSize;
			}
		}
	}

	std::swap(m_numericMatrix, renewedNumericMatrix);
	m_rowSize -= removedColumnSize;
}

void NumericalAnalysis::LinearAlgebra::NumericMatrix::eraseRows(std::initializer_list<size_type> rowIndices)
{
	std::vector<size_type> sortedRemovedRowIndices;
	{
		std::vector<size_type> originalRemovedRowIndices{ rowIndices };
		std::sort(originalRemovedRowIndices.begin(), originalRemovedRowIndices.end());
		std::unique_copy(originalRemovedRowIndices.begin(), originalRemovedRowIndices.end(), std::back_inserter(sortedRemovedRowIndices));

		if (sortedRemovedRowIndices.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The function of \"removeRow\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
			errorMessage += "Message : std::initializer_list<size_type> is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
		{
			if (m_columnSize <= sortedRemovedRowIndices.size())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Logic error\n";
				errorMessage += "Location: The function of \"removeRow\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : It is impossible to execute this function since m_columnSize is not more than the size of sortedRemovedRowIndices.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::logic_error(errorMessage);
			}

			if (m_columnSize <= sortedRemovedRowIndices.back())
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Out of range\n";
				errorMessage += "Location: The function of \"removeRow\" in the class of NumericalAnalysis::LinearAlgebra::NumericMatrix.\n";
				errorMessage += "Message : removedRowIndices contains an index being out of range.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::out_of_range(errorMessage);
			}
		}
	}

	const size_type renewedColumnSize = m_columnSize - sortedRemovedRowIndices.size();
	std::valarray<value_type> renewedNumericMatrix(renewedColumnSize * m_rowSize);

	pointer renewedNumericMatrixPointer = std::begin(renewedNumericMatrix);
	const_pointer numericMatrixPointer = std::begin(m_numericMatrix);

	sortedRemovedRowIndices.push_back(0);
	std::vector<size_type>::const_iterator sortedRemovedIndicesIter = sortedRemovedRowIndices.begin();

	for (size_type rowIndex = 0; rowIndex < m_columnSize; ++rowIndex)
	{
		if (rowIndex == *sortedRemovedIndicesIter)
		{
			++numericMatrixPointer;
			++sortedRemovedIndicesIter;
		}

		else
		{
			cblas_dcopy(m_rowSize, numericMatrixPointer, m_columnSize, renewedNumericMatrixPointer, renewedColumnSize);
			++renewedNumericMatrixPointer;
			++numericMatrixPointer;
		}
	}

	std::swap(m_numericMatrix, renewedNumericMatrix);
	m_columnSize = renewedColumnSize;
}

// Customize functions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

void NumericMatrix::insertColumn(const size_type insertedColumnIndex, const std::valarray<value_type>& va, const size_type additionalRowSize)
{
	std::valarray<value_type> renewedNumericMatrix(m_columnSize * (m_rowSize + additionalRowSize));
	const size_type leftCopySize = (m_columnSize * insertedColumnIndex);
	pointer rightCopyStart = std::begin(renewedNumericMatrix) + leftCopySize + (m_columnSize * additionalRowSize);

	cblas_dcopy(m_columnSize * additionalRowSize, std::begin(va), 1, std::begin(renewedNumericMatrix) + leftCopySize, 1);

	if (0 < leftCopySize)
	{
		cblas_dcopy(leftCopySize, std::begin(m_numericMatrix), 1, std::begin(renewedNumericMatrix), 1);
	}

	if (insertedColumnIndex < m_rowSize)
	{
		cblas_dcopy(m_columnSize, std::begin(m_numericMatrix) + leftCopySize, 1, rightCopyStart, 1);
	}

	std::swap(m_numericMatrix, renewedNumericMatrix);
	m_rowSize += additionalRowSize;
}

void NumericMatrix::insertRow(const size_type insertedRowIndex, const std::valarray<value_type>& va, const size_type additionalColumnSize)
{
	const size_type renewedColumnSize = m_columnSize + additionalColumnSize;
	std::valarray<value_type> renewedNumericMatrix(renewedColumnSize * m_rowSize);

	size_type copyIndex = 0;
	size_type insertIndex = 0;

	while (insertIndex < insertedRowIndex)
	{
		cblas_dcopy(m_rowSize, (std::begin(m_numericMatrix) + copyIndex), m_columnSize, std::begin(renewedNumericMatrix) + insertIndex, renewedColumnSize);
		++copyIndex;
		++insertIndex;
	}

	for (size_type pointerIndex = 0; pointerIndex < additionalColumnSize; ++pointerIndex)
	{
		cblas_dcopy(m_rowSize, (std::begin(va) + pointerIndex), additionalColumnSize, std::begin(renewedNumericMatrix), renewedColumnSize);
		++insertIndex;
	}

	while (insertIndex < m_rowSize)
	{
		cblas_dcopy(m_rowSize, (std::begin(m_numericMatrix) + copyIndex), m_columnSize, std::begin(renewedNumericMatrix) + insertIndex, renewedColumnSize);
		++copyIndex;
		++insertIndex;
	}

	std::swap(m_numericMatrix, renewedNumericMatrix);
	m_columnSize = renewedColumnSize;
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods


