#include "NumericVector.h"

#include "NumericVectorSlice.h"

using namespace NumericalAnalysis::LinearAlgebra;


// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors

NumericVector::NumericVector(std::initializer_list<value_type> init)
	: m_numericVector(init)
{
}

NumericVector::NumericVector(const size_type size)
	: m_numericVector(size)
{
}

NumericVector::NumericVector(const value_type value, const size_type size)
	: m_numericVector(value, size)
{
	if (size == 0)
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Invalid argument\n";
		errorMessage += "Location: The constructor of the class of NumericalAnalysis::LinearAlgebra::NumericVector.\n";
		errorMessage += "Message : size is zero. It is impossible for this constructor to create an object with empty m_numericVector.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::invalid_argument(errorMessage);
	}
}

NumericVector::NumericVector(const NumericVector& numericVector)
	: m_numericVector(numericVector.size())
{
	cblas_dcopy(m_numericVector.size(), numericVector.begin(), 1, std::begin(m_numericVector), 1);
}

NumericVector::NumericVector(const NumericVectorSlice& numericVectorSlice)
	: m_numericVector(numericVectorSlice.m_size)
{
	cblas_dcopy(m_numericVector.size(), numericVectorSlice.begin(), numericVectorSlice.m_stride, std::begin(m_numericVector), 1);
}

// Constructors
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Arithmetic functions

NumericVector& NumericVector::add(const value_type scalingMultiplier, const NumericVectorSlice& numericVectorSlice) noexcept
{
	cblas_daxpy(m_numericVector.size(), scalingMultiplier, numericVectorSlice.begin(), numericVectorSlice.m_stride, std::begin(m_numericVector), 1);
	return *this;
}

NumericVector& NumericVector::add(const value_type scalingMultiplier, const NumericVectorSlice& numericVectorSlice, const value_type selfScalingMultiplier) noexcept
{
	cblas_daxpby(m_numericVector.size(), scalingMultiplier, numericVectorSlice.begin(), numericVectorSlice.m_stride, selfScalingMultiplier, std::begin(m_numericVector), 1);
	return *this;
}

// Arithmetic functions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Unary operators

NumericVector& NumericVector::operator=(const NumericVectorSlice& numericVectorSlice) noexcept
{
	m_numericVector.resize(numericVectorSlice.m_size);
	cblas_dcopy(m_numericVector.size(), numericVectorSlice.begin(), numericVectorSlice.m_stride, std::begin(m_numericVector), 1);
	return *this;
}

NumericVector& NumericVector::operator+=(const NumericVectorSlice& numericVectorSlice) noexcept
{
	return add(1.0, numericVectorSlice);
}

NumericVector& NumericVector::operator-=(const NumericVectorSlice& numericVectorSlice) noexcept
{
	return add(-1.0, numericVectorSlice);
}

NumericVector& NumericVector::operator*=(const NumericVectorSlice& numericVectorSlice) noexcept
{
	std::valarray<value_type> temp = std::move(m_numericVector);
	m_numericVector.resize(temp.size());
	vmdMulI(m_numericVector.size(), std::begin(temp), 1, numericVectorSlice.begin(), numericVectorSlice.m_stride, std::begin(m_numericVector), 1, VML_HA);
	return *this;
}

NumericVector& NumericVector::operator/=(const NumericVectorSlice& numericVectorSlice) noexcept
{
	std::valarray<value_type> temp = std::move(m_numericVector);
	m_numericVector.resize(temp.size());
	vmdDivI(m_numericVector.size(), std::begin(temp), 1, numericVectorSlice.begin(), numericVectorSlice.m_stride, std::begin(m_numericVector), 1, VML_HA);
	return *this;
}

// Unary operators
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Slice functions

NumericVectorSlice NumericVector::operator[](const std::slice& sl) noexcept
{
	return NumericVectorSlice(m_numericVector, sl);
}

// Slice functions
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Access functions
