#include "Receptionist.h"

#include <iostream>
#include <sstream>
#include <fstream>
#include <iterator>
#include <regex>
#include <vector>

#include "UnitConversion.h"

#include "PeriodicTable.h"
#include "ChemicalConstituents.h"
#include "CrystalLattice.h"

using namespace ControlCenter::Window;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Public methods

void Receptionist::sendOrderSheetManual()
{
	std::string outputString;
	outputString += "---------------------------------------------------------------------------------\n\n";
	outputString += "Manual for order sheet\n\n";
	outputString += "---------------------------------------------------------------------------------\n\n";


	std::cout << outputString << std::endl;
}

void Receptionist::sendCommandLineArgumentsManual()
{
	std::string outputString;
	outputString += "---------------------------------------------------------------------------------\n\n";
	outputString += "Manual for command-line arguments\n\n";
	outputString += "---------------------------------------------------------------------------------\n\n";

	outputString += "------------------------------------------------------\n\n";
	outputString += "Usage   : MATEVENT.exe [options] file1 [...]\n";
	outputString += "------------------------------------------------------\n\n";

	outputString += "------------------------------------------------------\n";
	outputString += "#\n";
	outputString += "# Options\n";
	outputString += "#\n\n";
	outputString += "-osp    : Flag for the path to the order sheet.\n";
	outputString += "------------------------------------------------------\n\n";

	outputString += "------------------------------------------------------\n";
	outputString += "#\n";
	outputString += "# Options\n";
	outputString += "#\n\n";
	outputString += "-mpin   : The index of this mpi.\n";
	outputString += "------------------------------------------------------\n\n";

	outputString += "------------------------------------------------------\n";
	outputString += "#\n";
	outputString += "# Options\n";
	outputString += "#\n\n";
	outputString += "-node   : The number of nudes.\n";
	outputString += "------------------------------------------------------\n\n";

	outputString += "------------------------------------------------------\n";
	outputString += "#\n";
	outputString += "# Otherwise\n";
	outputString += "#\n\n";
	outputString += "--help  : Print Help and exit\n";
	outputString += "------------------------------------------------------\n";

	std::cout << outputString << std::endl;
}

// Public methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods

std::string Receptionist::loadOrderSheet(const std::filesystem::path& orderSheetPath) const
{
	std::string orderSheetText;
	{
		if (orderSheetPath.empty())
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Invalid argument\n";
			errorMessage += "Location: The member function of \"loadOrderSheet\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : orderSheetPath is empty.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::invalid_argument(errorMessage);
		}

		else
		{
			std::ifstream inputFileStream;
			inputFileStream.open(orderSheetPath, std::ios_base::in);

			if (!inputFileStream)
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"loadOrderSheet\" in the class of ControlCenter::Window::Receptionist.\n";
				errorMessage += "Message : Could not open the order sheet of ";
				{
					errorMessage += orderSheetPath.generic_string();
					errorMessage += ".\n\n";
				}
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}

			else
			{
				for (std::string stringLine; std::getline(inputFileStream, stringLine);)
					orderSheetText += stringLine + "\n";

				inputFileStream.close();
			}
		}
	}

	return orderSheetText;
}

void Receptionist::readExecutionEnvironment(const std::string& orderSheetText)
{
	{
		std::regex pat("Order.Name[\\s]+([[:alnum:]:/\\-\\_]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			m_orderForm.resetOrderName(sm.str(1));
		else
			m_orderForm.resetOrderName();
	}
	{
		std::regex pat("Execution.Mode[\\s]+([[:alnum:]]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			m_orderForm.resetExecutionMode(sm.str(1));
		else
			m_orderForm.resetExecutionMode();
	}
	{
		std::regex pat("Max.Threading[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			m_orderForm.resetMaxThreading(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			m_orderForm.resetMaxThreading();
	}
	{
		std::regex pat("Mkl.Threading[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			m_orderForm.resetMklThreading(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			m_orderForm.resetMklThreading();
	}
}

void Receptionist::readIOPaths(const std::string& orderSheetText)
{
	{
		std::regex pat("Output.Directory.Path[\\s]+([[:alnum:]:/\\-\\.\\_]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			m_orderForm.resetOutputDirectoryPath(std::filesystem::path{ sm.str(1) });

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readIOPaths\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Output.Directory.Path.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Work.Directory.Path[\\s]+([[:alnum:]:/\\-\\.\\_]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			m_orderForm.resetWorkDirectoryPath(std::filesystem::path{ sm.str(1) });

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readIOPaths\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Work.Directory.Path.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
}

void Receptionist::readMonitoringPlan(const std::string& orderSheetText)
{
	MonitoringPlan monitoringPlan = m_orderForm.monitoringPlan();
	{
		std::regex pat("MD.Record.Frequency[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			monitoringPlan.resetMdRecordFrequency(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			monitoringPlan.resetMdRecordFrequency();
	}
	{
		std::regex pat("Chemical.Bonding.Record.Frequency[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			monitoringPlan.resetChemicalBondingRecordFrequency(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			monitoringPlan.resetChemicalBondingRecordFrequency();
	}
	{
		std::regex pat("Record.All.Success.Sample.Data[\\s]+([[:alpha:]]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			monitoringPlan.resetAllSuccessSampleDataNecessity(getSwitch(sm.str(1)));

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readMonitoringPlan\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Record.All.Success.Sample.Data.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Record.Failure.Data[\\s]+([[:alpha:]]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			monitoringPlan.resetFailureDataNecessity(getSwitch(sm.str(1)));

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readMonitoringPlan\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Record.Failure.Data.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Final.Simulations.Reports.Per.Structure[\\s]+([[:alpha:]]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
			monitoringPlan.resetFinalSimulationsReportsPerStructureNecessity(getSwitch(sm.str(1)));

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readMonitoringPlan\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Final.Simulations.Reports.Per.Structure.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}

	m_orderForm.resetMonitoringPlan(std::move(monitoringPlan));
}

void Receptionist::readVertexSharingCationicPairs(const std::string& orderSheetText)
{
	std::size_t initialPos = orderSheetText.find("<Vertex.Sharing.Cationic.Pairs");
	std::size_t endPos = orderSheetText.find("Vertex.Sharing.Cationic.Pairs>");

	if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
	{
		std::size_t tempPos = orderSheetText.find_first_of("\n", initialPos);
		std::string cationicPairsString{ orderSheetText, (tempPos + 1), (endPos - tempPos - 1) };

		CrystallizationPlan crystallizationPlan = m_orderForm.crystallizationPlan();
		crystallizationPlan.resetVertexSharingCationicPairs(readCationicPairs(cationicPairsString));
		m_orderForm.resetCrystallizationPlan(std::move(crystallizationPlan));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readVertexSharingCationicPairs\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read Vertex.Sharing.Cationic.Pairs.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

void Receptionist::readEdgeSharingCationicPairs(const std::string& orderSheetText)
{
	std::size_t initialPos = orderSheetText.find("<Edge.Sharing.Cationic.Pairs");
	std::size_t endPos = orderSheetText.find("Edge.Sharing.Cationic.Pairs>");

	if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
	{
		std::size_t tempPos = orderSheetText.find_first_of("\n", initialPos);
		std::string cationicPairsString{ orderSheetText, (tempPos + 1), (endPos - tempPos - 1) };

		CrystallizationPlan crystallizationPlan = m_orderForm.crystallizationPlan();
		crystallizationPlan.resetEdgeSharingCationicPairs(readCationicPairs(cationicPairsString));
		m_orderForm.resetCrystallizationPlan(std::move(crystallizationPlan));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readEdgeSharingCationicPairs\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read Edge.Sharing.Cationic.Pairs.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

void Receptionist::readFaceSharingCationicPairs(const std::string& orderSheetText)
{
	std::size_t initialPos = orderSheetText.find("<Face.Sharing.Cationic.Pairs");
	std::size_t endPos = orderSheetText.find("Face.Sharing.Cationic.Pairs>");

	if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
	{
		std::size_t tempPos = orderSheetText.find_first_of("\n", initialPos);
		std::string cationicPairsString{ orderSheetText, (tempPos + 1), (endPos - tempPos - 1) };

		CrystallizationPlan crystallizationPlan = m_orderForm.crystallizationPlan();
		crystallizationPlan.resetFaceSharingCationicPairs(readCationicPairs(cationicPairsString));
		m_orderForm.resetCrystallizationPlan(std::move(crystallizationPlan));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readFaceSharingCationicPairs\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read Face.Sharing.Cationic.Pairs.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

void Receptionist::readCommonOptimizationVariables(const std::string& orderSheetText)
{
	CrystallizationPlan crystallizationPlan = m_orderForm.crystallizationPlan();
	{
		std::regex pat("Maximum.Bonding.Length.Error.Rate[\\s]+([e\\d\\.]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
		{
			double maxBondingLengthErrorRate = 0.0;
			std::stringstream maxBondingLengthErrorRateStream{ sm.str(1) };
			maxBondingLengthErrorRateStream >> maxBondingLengthErrorRate;
			crystallizationPlan.resetMaxBondingLengthErrorRate(maxBondingLengthErrorRate);
		}

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readCrystallizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Maximum.Bonding.Length.Error.Rate.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Minimum.Packing.Fraction[\\s]+([e\\d\\.]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
		{
			double minPackingFraction = 0.0;
			std::stringstream minPackingFractionStream{ sm.str(1) };
			minPackingFractionStream >> minPackingFraction;
			crystallizationPlan.resetMinPackingFraction(minPackingFraction);
		}

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readCrystallizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Minimum.Packing.Fraction.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Pressure[\\s]+([e\\d\\.]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
		{
			double pressure = 0.0;
			std::stringstream pressureStream{ sm.str(1) };
			pressureStream >> pressure;
			crystallizationPlan.resetPressure(pressure);
		}

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readCrystallizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Pressure.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Attractive.Force.Constant[\\s]+([e\\d\\.\\-]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
		{
			double attractiveForceConstant = 0.0;
			std::stringstream attractiveForceConstantStream{ sm.str(1) };
			attractiveForceConstantStream >> attractiveForceConstant;
			crystallizationPlan.resetAttractiveForceConstant(attractiveForceConstant);
		}

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readCrystallizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Attractive.Force.Constant.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
	{
		std::regex pat("Repulsive.Force.Constant[\\s]+([e\\d\\.\\-]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(orderSheetText, sm, pat))
		{
			double repulsiveForceConstant = 0.0;
			std::stringstream repulsiveForceConstantStream{ sm.str(1) };
			repulsiveForceConstantStream >> repulsiveForceConstant;
			crystallizationPlan.resetRepulsiveForceConstant(repulsiveForceConstant);
		}

		else
		{
			std::string errorMessage = "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readCrystallizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : Could not read Repulsive.Force.Constant.\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}

	m_orderForm.resetCrystallizationPlan(std::move(crystallizationPlan));
}

void Receptionist::readGlobalOptimizationVariables(const std::string& orderSheetText)
{
	std::size_t initialPos = orderSheetText.find("<Global.Optimization.Variables");
	std::size_t endPos = orderSheetText.find("Global.Optimization.Variables>");

	if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
	{
		std::size_t tempPos = orderSheetText.find_first_of("\n", initialPos);
		std::string globalOptimizationVariablesString{ orderSheetText, (tempPos + 1), (endPos - tempPos - 1) };

		CrystallizationPlan crystallizationPlan = m_orderForm.crystallizationPlan();
		crystallizationPlan.resetGlobalOptimizationVariables(readCrystallizationVariables(globalOptimizationVariablesString));
		m_orderForm.resetCrystallizationPlan(std::move(crystallizationPlan));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readGlobalOptimizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read Global.Optimization.Variables.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

void Receptionist::readLocalOptimizationVariables(const std::string& orderSheetText)
{
	std::size_t initialPos = orderSheetText.find("<Local.Optimization.Variables");
	std::size_t endPos = orderSheetText.find("Local.Optimization.Variables>");

	if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
	{
		std::size_t tempPos = orderSheetText.find_first_of("\n", initialPos);
		std::string localOptimizationVariablesString{ orderSheetText, (tempPos + 1), (endPos - tempPos - 1) };

		CrystallizationPlan crystallizationPlan = m_orderForm.crystallizationPlan();
		crystallizationPlan.resetLocalOptimizationVariables(readCrystallizationVariables(localOptimizationVariablesString));
		m_orderForm.resetCrystallizationPlan(std::move(crystallizationPlan));
	}

	else
	{
		std::string errorMessage = "------------------------------------------------------\n\n";
		errorMessage += "Error   : Runtime error\n";
		errorMessage += "Location: The member function of \"readLocalOptimizationVariables\" in the class of ControlCenter::Window::Receptionist.\n";
		errorMessage += "Message : Could not read Local.Optimization.Variables.\n\n";
		errorMessage += "------------------------------------------------------\n\n";
		throw std::runtime_error(errorMessage);
	}
}

void Receptionist::readRequestedFeedstocks(const std::string& orderSheetText)
{
	if (m_orderForm.shouldGenerateMaterials())
	{
		size_type feedstocksPerChemicalConstituents = 0;
		{
			std::regex pat("Feedstocks.Per.Chemical.Constituents[\\s]+([\\d]+)([\\n]|[\\s])");
			std::smatch sm;

			if (std::regex_search(orderSheetText, sm, pat))
				feedstocksPerChemicalConstituents = static_cast<size_type>(std::stoi(sm.str(1)));
		}

		std::vector<SolidStatePhysics::Materials::ChemicalConstituents> targetChemicalConstituentsCollection;
		{
			std::size_t initialPos = orderSheetText.find("<Chemical.Constituents.Collection");
			std::size_t endPos = orderSheetText.find("Chemical.Constituents.Collection>");

			if (initialPos != std::string::npos && endPos != std::string::npos && initialPos < endPos)
			{
				std::size_t tempPos = orderSheetText.find_first_of("\n", initialPos);
				std::string constituentsString{ orderSheetText, (tempPos + 1), (endPos - tempPos - 1) };
				std::stringstream constituentsStream{ constituentsString };

				std::regex totalPat("[\\s]*([[:alpha:]]+[\\s]+[\\d]+[\\s]+)*([[:alpha:]]+[\\s]+[\\d]+){1}([\\s])*");
				std::regex pat("([[:alpha:]]+)[\\s]+([\\d]+)");

				for (std::string stringLine; std::getline(constituentsStream, stringLine);)
				{
					std::smatch totalSm;

					if (std::regex_match(stringLine, totalSm, totalPat))
					{
						SolidStatePhysics::Materials::ChemicalConstituents chemicalConstituents;

						for (std::sregex_iterator iter(stringLine.begin(), stringLine.end(), pat); iter != std::sregex_iterator{}; ++iter)
							chemicalConstituents.tryEmplaceAtoms(SolidStatePhysics::ChemicalElements::PeriodicTable::replyAtomicNumber(iter->str(1)), static_cast<unsigned int>(std::stoi(iter->str(2))));

						targetChemicalConstituentsCollection.push_back(std::move(chemicalConstituents));
					}

					else
						continue;
				}
			}
		}

		try
		{
			ControlCenter::Administering::RequestedFeedstocks requestedFeedstocks;

			requestedFeedstocks.resetFeedstocksPerChemicalConstituents(feedstocksPerChemicalConstituents);
			requestedFeedstocks.resetTargetChemicalConstituents(std::move(targetChemicalConstituentsCollection));
			m_orderForm.resetRequestedFeedstocks(std::move(requestedFeedstocks));
		}

		catch (const std::logic_error& logicError)
		{
			std::string errorMessage = logicError.what();
			errorMessage += "------------------------------------------------------\n\n";
			errorMessage += "Error   : Logic error\n";
			errorMessage += "Location: The member function of \"readRequestedFeedstocks\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : This function catches the logic_error from the function of \"orderForm\".\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::logic_error(errorMessage);
		}

		catch (const std::runtime_error& runtimeError)
		{
			std::string errorMessage = runtimeError.what();
			errorMessage += "------------------------------------------------------\n\n";
			errorMessage += "Error   : Runtime error\n";
			errorMessage += "Location: The member function of \"readRequestedFeedstocks\" in the class of ControlCenter::Window::Receptionist.\n";
			errorMessage += "Message : This function catches the runtime_error from the function of \"orderForm\".\n\n";
			errorMessage += "------------------------------------------------------\n\n";
			throw std::runtime_error(errorMessage);
		}
	}
}

// Private methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Private methods for reading methods

SolidStatePhysics::Crystallization::Simulators::CrystallizationVariables Receptionist::readCrystallizationVariables(const std::string& inputText)
{
	CrystallizationVariables crystallizationVariables;
	{
		std::regex pat("Necessary.Geometrical.Optimizing[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(inputText, sm, pat))
			crystallizationVariables.resetNecessaryGeometricalOptimizing(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			crystallizationVariables.resetNecessaryGeometricalOptimizing();
	}
	{
		std::regex pat("Geometrical.Constraints.Lifetime[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(inputText, sm, pat))
			crystallizationVariables.resetGeometricalConstraintsLifetime(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			crystallizationVariables.resetGeometricalConstraintsLifetime();
	}
	{
		std::regex pat("Linked.Polyhedra.Lifetime[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(inputText, sm, pat))
			crystallizationVariables.resetLinkedPolyhedraLifetime(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			crystallizationVariables.resetLinkedPolyhedraLifetime();
	}
	{
		std::regex pat("Crystal.Lattice.Lifetime[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(inputText, sm, pat))
			crystallizationVariables.resetCrystalLatticeLifetime(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			crystallizationVariables.resetCrystalLatticeLifetime();
	}
	{
		std::regex pat("Thermal.Transformation.Frequency[\\s]+([\\d]+)([\\n]|[\\s])");
		std::smatch sm;

		if (std::regex_search(inputText, sm, pat))
			crystallizationVariables.resetThermalTransformationFrequency(static_cast<std::size_t>(std::stoi(sm.str(1))));
		else
			crystallizationVariables.resetThermalTransformationFrequency();
	}
	{
		double initialMaxDisplacement = 0.0;
		{
			std::regex pat("Initial.Maximum.Displacement[\\s]+([e\\d\\.]+)([\\n]|[\\s])");
			std::smatch sm;
			
			if (std::regex_search(inputText, sm, pat))
			{
				double initialMaxDisplacementAng = 0.0;
				std::stringstream initialMaxDisplacementStream{ sm.str(1) };
				initialMaxDisplacementStream >> initialMaxDisplacementAng;
				initialMaxDisplacement = SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(initialMaxDisplacementAng);
			}

			else
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"readCrystallizationFactors\" in the class of ControlCenter::Window::Receptionist.\n";
				errorMessage += "Message : Could not read Initial.Maximum.Displacement.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}
		}

		double finalMaxDisplacement = 0.0;
		{
			std::regex pat("Final.Maximum.Displacement[\\s]+([e\\d\\.]+)([\\n]|[\\s])");
			std::smatch sm;

			if (std::regex_search(inputText, sm, pat))
			{
				double finalMaxDisplacementAng = 0.0;
				std::stringstream finalMaxDisplacementStream{ sm.str(1) };
				finalMaxDisplacementStream >> finalMaxDisplacementAng;
				finalMaxDisplacement = SolidStatePhysics::UnitSystem::length_cast<SolidStatePhysics::UnitSystem::LengthUnit::AtomicUnit>{}(finalMaxDisplacementAng);
			}

			else
			{
				std::string errorMessage = "------------------------------------------------------\n\n";
				errorMessage += "Error   : Runtime error\n";
				errorMessage += "Location: The member function of \"readCrystallizationFactors\" in the class of ControlCenter::Window::Receptionist.\n";
				errorMessage += "Message : Could not read Final.Maximum.Displacement.\n\n";
				errorMessage += "------------------------------------------------------\n\n";
				throw std::runtime_error(errorMessage);
			}
		}

		crystallizationVariables.resetMaxDisplacements(initialMaxDisplacement, finalMaxDisplacement);
	}

	return crystallizationVariables;
}

std::set<std::pair<std::string, std::string>> Receptionist::readCationicPairs(const std::string& inputText)
{
	std::set<std::pair<std::string, std::string>> cationicPairs;
	{
		std::stringstream cationicPairsStream{ inputText };
		std::regex pat("[\\s]*([[:alpha:]]+){1}[\\s]+([[:alpha:]]+){1}([\\s])*");

		for (std::string stringLine; std::getline(cationicPairsStream, stringLine);)
		{
			std::smatch sm;

			if (std::regex_match(stringLine, sm, pat))
				cationicPairs.emplace(std::make_pair(sm.str(1), sm.str(2)));
			else
				continue;
		}
	}

	return cationicPairs;
}

// Private methods for reading methods
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
