#include "SecondNeighborConstrainter.h"

using namespace SolidStatePhysics::Crystallization::Components;

// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// Constructors and destructor

SecondNeighborConstrainter::SecondNeighborConstrainter(SecondNeighborConstraintOwner& realOwner, const SecondNeighborConstraintOwnerAdaptor& attachedOwner)
	: MinimumConstrainter{ realOwner.ownerAtom(), AtomicAdaptor(attachedOwner.entity().ownerAtom(), attachedOwner) }
	, m_realOwner{ std::ref(realOwner) }
	, m_attachedOwner{ attachedOwner }
{
}

SecondNeighborConstrainter::SecondNeighborConstrainter(SecondNeighborConstraintOwner& realOwner, SecondNeighborConstraintOwnerAdaptor&& attachedOwner)
	: MinimumConstrainter{ realOwner.ownerAtom(), AtomicAdaptor(attachedOwner.entity().ownerAtom(), attachedOwner) }
	, m_realOwner{ std::ref(realOwner) }
	, m_attachedOwner{ std::move(attachedOwner) }
{
}

SecondNeighborConstrainter::SecondNeighborConstrainter(const std::shared_ptr<SecondNeighborConstrainter>& twinPtr)
	: MinimumConstrainter{ std::static_pointer_cast<MinimumConstrainter>(twinPtr) }
	, m_realOwner{ std::ref(twinPtr->m_attachedOwner.entity()) }
	, m_attachedOwner{ twinPtr->m_realOwner, twinPtr->m_attachedOwner }
{
	m_attachedOwner.reverseLatticePoint();
}

SecondNeighborConstrainter::~SecondNeighborConstrainter()
{
	if (std::shared_ptr<SecondNeighborConstrainter> twinSecondNeighborConstrainterPtr = std::dynamic_pointer_cast<SecondNeighborConstrainter>(twinPtr()))
	{
		twinSecondNeighborConstrainterPtr->m_realOwner.get().disjoin();
		twinSecondNeighborConstrainterPtr->resetTwinPtr();
	}
}

// Constructors and destructor
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
// **********************************************************************************************************************************************************************************************************************************************************************************************
